SUBROUTINE CalculateStats

USE Parameters
USE Globals
USE Procedures

IMPLICIT NONE

integer                 :: it
real(8), dimension(7)   :: whichperc

!note this is done with net expenditures (csim) but perhaps should be gross expenditures (cgrosim)

!Mean profiles
Ec      = SUM(csim, DIM=1)/real(nsim)
Ek      = SUM(ksim(:,1:Ttot), DIM=1)/real(nsim)
Pk0neg  = COUNT(ksim(:,1:Ttot)<=0.0, DIM=1)/real(nsim)
Eh      = SUM(hsim, DIM=1)/real(nsim)
Ehtot   = SUM(htotsim, DIM=1)/real(nsim)
Ehbar   = SUM(hbarsim, DIM=1)/real(nsim)
Phpos   = COUNT(hsim>hminpos, DIM=1)/real(nsim)
Phft    = COUNT(hsim>hminft, DIM=1)/real(nsim)
Ehpos   = SUM(hsim, MASK=hsim>hminpos, DIM=1)/real(COUNT(hsim>hminpos, DIM=1))
Ehft    = SUM(hsim, MASK=hsim>hminft, DIM=1)/real(COUNT(hsim>hminft, DIM=1))
Ew      = SUM(wsim, DIM=1)/real(nsim)
Ew_hpos = SUM(wsim, MASK=hsim>hminpos, DIM=1)/real(COUNT(hsim>hminpos, DIM=1))
Ey      = SUM(ysim(:,1:Twork), DIM=1)/real(nsim)
Ep      = SUM(psim, DIM=1)/real(nsim)

!write(*,*) 'Mean wage check:', sum(Ew)/real(38)

!Mean log profiles: ISSUE HERE SINCE 
Elogh       = SUM(dlog(hsim), MASK=hsim>hminpos, DIM=1)/real(COUNT(hsim>hminpos, DIM=1))
Eloghtot    = SUM(dlog(htotsim), MASK=htotsim>hminpos, DIM=1)/real(COUNT(htotsim>hminpos, DIM=1))
Eloghbar    = SUM(dlog(hbarsim), MASK=hbarsim>hminpos, DIM=1)/real(COUNT(hbarsim>hminpos, DIM=1))
Elogh_ft    = SUM(dlog(hsim), MASK=hsim>hminft, DIM=1)/real(COUNT(hsim>hminft, DIM=1))
Elogc       = SUM(dlog(csim), DIM=1)/real(nsim)
Elogc_hpos  = SUM(dlog(csim(:,1:Twork)), MASK=hsim>hminpos, DIM=1)/real(COUNT(hsim>hminpos, DIM=1))
Elogw       = SUM(dlog(wsim), DIM=1)/real(nsim)
Elogw_hpos  = SUM(dlog(wsim), MASK=hsim>hminpos, DIM=1)/real(COUNT(hsim>hminpos, DIM=1))
Elogy       = SUM(dlog(ysim(:,1:Twork)), MASK=hsim(:,1:Twork)>hminpos, DIM=1)/real(COUNT(hsim(:,1:Twork)>hminpos, DIM=1))
Elogp       = SUM(dlog(psim), MASK=psim>0.0, DIM=1)/real(COUNT(psim>0.0, DIM=1))

! IF(InitWealthDist==1) THEN !else fails at t=1 since all k=0
	Elogh_kpos  = SUM(dlog(hsim), MASK=ksim(:,1:Twork)>0.0 .and. hsim>hminpos, DIM=1)/real(COUNT(ksim(:,1:Twork)>0.0 .and. hsim>hminpos, DIM=1))
	Eloghtot_kpos  = SUM(dlog(htotsim), MASK=ksim(:,1:Twork)>0.0 .and. htotsim>hminpos, DIM=1)/real(COUNT(ksim(:,1:Twork)>0.0 .and. htotsim>hminpos, DIM=1))
	Elogw_kpos  = SUM(dlog(wsim), MASK=ksim(:,1:Twork)>0.0, DIM=1)/real(COUNT(ksim(:,1:Twork)>0.0, DIM=1))
	Elogw_khpos = SUM(dlog(wsim), MASK=ksim(:,1:Twork)>0.0 .and. hsim>hminpos, DIM=1)/real(COUNT(ksim(:,1:Twork)>0.0 .and. hsim>hminpos, DIM=1))
	Elogk       = SUM(dlog(abs(ksim(:,1:Ttot))), MASK=ksim(:,1:Ttot)>0.0, DIM=1) /real(MERGE(COUNT(ksim(:,1:Ttot)>0.0, DIM=1),1,COUNT(ksim(:,1:Ttot)>0.0, DIM=1)>0))	!merge is to avoid errors when the count is zero
	Elogk_hpos  = SUM(dlog(abs(ksim(:,1:Twork))), MASK=ksim(:,1:Twork)>0.0 .and. hsim>hminpos, DIM=1)/real(MERGE(COUNT(ksim(:,1:Twork)>0.0 .and. hsim>hminpos, DIM=1),1,COUNT(ksim(:,1:Twork)>0.0 .and. hsim>hminpos, DIM=1)>0))
! END IF

!Variance of raw
Vh          = SUM(hsim**2, DIM=1)/real(nsim) - Eh**2
Vk          = SUM(ksim(:,1:Ttot)**2, DIM=1)/real(nsim) - Ek**2

!Gini coefficient
DO it = 1,Ttot
	CALL Gini(nsim,ksim(:,it),GINIk(it))
END DO

!Variance of logs
Vlogh       = SUM(dlog(hsim)**2, MASK=hsim>hminpos, DIM=1)/real(COUNT(hsim>hminpos, DIM=1)) - Elogh**2
Vloghtot    = SUM(dlog(htotsim)**2, MASK=htotsim>hminpos, DIM=1)/real(COUNT(htotsim>hminpos, DIM=1)) - Eloghtot**2
Vloghbar    = SUM(dlog(hbarsim)**2, MASK=hbarsim>hminpos, DIM=1)/real(COUNT(hbarsim>hminpos, DIM=1)) - Eloghbar**2
Vlogh_ft    = SUM(dlog(hsim)**2, MASK=hsim>hminft, DIM=1)/real(COUNT(hsim>hminft, DIM=1)) - Elogh_ft**2
Vlogc       = SUM(dlog(csim)**2, DIM=1)/real(nsim) - Elogc**2
Vlogc_hpos  = SUM(dlog(csim(:,1:Twork))**2, MASK=hsim>hminpos, DIM=1)/real(COUNT(hsim>hminpos, DIM=1)) - Elogc_hpos**2
Vlogw       = SUM(dlog(wsim)**2, DIM=1)/real(nsim) - Elogw**2
Vlogw_hpos  = SUM(dlog(wsim)**2, MASK=hsim>hminpos, DIM=1)/real(COUNT(hsim>hminpos, DIM=1)) - Elogw_hpos**2
Vlogy       = SUM(dlog(ysim(:,1:Twork))**2, MASK=ysim(:,1:Twork)>0.0, DIM=1)/real(COUNT(ysim(:,1:Twork)>0.0, DIM=1)) - Elogy**2
! IF(InitWealthDist==1) THEN !else fails at t=1 since all k=0
	Vlogk       = SUM(dlog(abs(ksim(:,1:Ttot)))**2, MASK=ksim(:,1:Ttot)>0.0, DIM=1)/real(MERGE(COUNT(ksim(:,1:Ttot)>0.0, DIM=1),1,COUNT(ksim(:,1:Ttot)>0.0, DIM=1)>0)) - Elogk**2
! END IF

!Covariances of logs
 COVwh_hpos  = SUM(dlog(wsim)*dlog(hsim), MASK=hsim>hminpos, DIM=1)/real(COUNT(hsim>hminpos, DIM=1)) - Elogw_hpos*Elogh
 COVwhtot_hpos  = SUM(dlog(wsim)*dlog(htotsim), MASK=htotsim>hminpos, DIM=1)/real(COUNT(htotsim>hminpos, DIM=1)) - Elogw_hpos*Eloghtot
 COVcw_hpos  = SUM(dlog(csim(:,1:Twork))*dlog(wsim), MASK=hsim>hminpos, DIM=1)/real(COUNT(hsim>hminpos, DIM=1)) - Elogc_hpos*Elogw_hpos
 COVcw       = SUM(dlog(csim(:,1:Twork))*dlog(wsim), DIM=1)/real(nsim) - Elogc(1:Twork)*Elogw
 COVch_hpos  = SUM(dlog(csim(:,1:Twork))*dlog(hsim), MASK=hsim>hminpos, DIM=1)/real(COUNT(hsim>hminpos, DIM=1)) - Elogc_hpos*Elogh
 COVchtot_hpos  = SUM(dlog(csim(:,1:Twork))*dlog(htotsim), MASK=htotsim>hminpos, DIM=1)/real(COUNT(htotsim>hminpos, DIM=1)) - Elogc_hpos*Eloghtot
 COVhhbar_hpos  = SUM(dlog(hsim(:,1:Twork))*dlog(hbarsim), MASK=htotsim>hminpos, DIM=1)/real(COUNT(htotsim>hminpos, DIM=1)) - Eloghbar*Elogh

! IF(InitWealthDist==1) THEN !else fails at t=1 since all k=0
	COVkw       = SUM(dlog(abs(ksim(:,1:Twork)))*dlog(wsim), MASK=ksim(:,1:Twork)>0.0, DIM=1)/real(MERGE(COUNT(ksim(:,1:Twork)>0.0, DIM=1),1,COUNT(ksim(:,1:Twork)>0.0, DIM=1)>0)) - Elogk(1:Twork)*Elogw_kpos
	COVkw_hpos  = SUM(dlog(abs(ksim(:,1:Twork)))*dlog(wsim), MASK=ksim(:,1:Twork)>0.0 .and. hsim>hminpos, DIM=1)/real(MERGE(COUNT(ksim(:,1:Twork)>0.0 .and. hsim>hminpos, DIM=1),1,COUNT(ksim(:,1:Twork)>0.0 .and. hsim>hminpos, DIM=1)>0)) - Elogk_hpos*Elogw_khpos
	COVkh_hpos  = SUM(dlog(abs(ksim(:,1:Twork)))*dlog(hsim), MASK=ksim(:,1:Twork)>0.0 .and. hsim>hminpos, DIM=1)/real(MERGE(COUNT(ksim(:,1:Twork)>0.0 .and. hsim>hminpos, DIM=1),1,COUNT(ksim(:,1:Twork)>0.0 .and. hsim>hminpos, DIM=1)>0)) - Elogk_hpos*Elogh_kpos
	COVkhtot_hpos  = SUM(dlog(abs(ksim(:,1:Twork)))*dlog(htotsim), MASK=ksim(:,1:Twork)>0.0 .and. htotsim>hminpos, DIM=1)/real(MERGE(COUNT(ksim(:,1:Twork)>0.0 .and. htotsim>hminpos, DIM=1),1,COUNT(ksim(:,1:Twork)>0.0 .and. htotsim>hminpos, DIM=1)>0)) - Elogk_hpos*Eloghtot_kpos
! END IF
  
!Percentiles of distributions
whichperc = (/ 0.05, 0.1, 0.25, 0.5, 0.75, 0.9, 0.95 /)
DO it = 1,Twork
    CALL Percentiles (nsim, csim(:,it), 7, whichperc, PCc(:,it))
    CALL Percentiles (nsim, ksim(:,it), 7, whichperc, PCk(:,it))
    CALL Percentiles (nsim, hsim(:,it), 7, whichperc, PCh(:,it))
    CALL Percentiles (nsim, wsim(:,it), 7, whichperc, PCw(:,it))
    CALL Percentiles (nsim, ysim(:,it), 7, whichperc, PCy(:,it))
END DO
END SUBROUTINE CalculateStats