#! /bin/sh                                                                                                             
#remember: cant have spaces between equals signs in assignments
#input (1) Debug/Release (2) name (3) run

DFLSPATH=/Volumes/FILES/Projects/Fortran/DFLS
DFLSFILE1=$DFLSPATH/newuoa-h.f
DFLSFILE2=$DFLSPATH/newuob-h.f
DFLSFILE3=$DFLSPATH/update.f
DFLSFILE4=$DFLSPATH/trsapp-h.f 
DFLSFILE5=$DFLSPATH/biglag.f
DFLSFILE6=$DFLSPATH/bigden.f

DFBOLSPATH=/Volumes/FILES/Projects/Fortran/DFBOLS
DFBOLSFILE1=$DFBOLSPATH/bobyqa_h.f
DFBOLSFILE2=$DFBOLSPATH/bobyqb_h.f
DFBOLSFILE3=$DFBOLSPATH/update.f
DFBOLSFILE4=$DFBOLSPATH/trsbox_h.f 
DFBOLSFILE5=$DFBOLSPATH/rescue_h.f
DFBOLSFILE6=$DFBOLSPATH/altmov.f
DFBOLSFILE7=$DFBOLSPATH/prelim_h.f

# Set up environment variables for Mac and Intel Fortan 11.0

source /opt/intel/Compiler/11.1/089/bin/ifortvars.sh intel64

#Debug version
if [ "$1" == "Debug" ]; then
	echo "Compiling in " $1 "mode..."
	
	# Delete existing files
	rm *.o *.mod
	
	# Compile and link debug version: produces all debugging output, no optimizations, for use with Fx3
	ifort -m64 -g -debug all -implicitnone -save-temps \
		-check all -warn all -fp-stack-check -ftrapuv \
	 -traceback -check noarg_temp_created \
		-gen-interfaces -warn interfaces -Wl,-stack_size,0x10000000 \
		Parameters.f90 Globals.f90 Procedures.f90  OtherCode/random.f90 \
		SetParameters.f90 LoadData.f90 Grids.f90 Estimation.f90 MomentConditions.f90	\
		Decisions.f90 Simulate.f90 DrawRandomNumbers.f90 \
		CalculateStats.f90 SaveOutput.f90 \
		Functions/FnAggHours.f90 Functions/FnBetaKY.f90 Functions/FnHoursBC.f90 Functions/FnSSParam.f90 Functions/dfovec.f90 \
		Functions/FnZgrid.f90 Functions/FnGridFixedEffect.f90 Functions/FnGridPrefHet.f90 Functions/FnGridPrefHetTrans.f90 \
		Functions/FnGridTrans.f90 Functions/FnTax.f90 \
		OtherCode/rtsecBetaKY.f90 OtherCode/rtsecHoursBC.f90 OtherCode/rtbisHoursBC.f90 OtherCode/rtflspHoursBC.f90 OtherCode/rtsecSSParam.f90 OtherCode/rtsecAggHours.f90 \
		OtherCode/minpack.f90 OtherCode/golden.f90 OtherCode/sort.f90 OtherCode/cumnor.f90 OtherCode/mnbrak.f90 OtherCode/rtbis.f90 \
		$DFLSFILE1 $DFLSFILE2 $DFLSFILE3 $DFLSFILE4 $DFLSFILE5 $DFLSFILE6 \
		Main.f90 -o $2.out \
		$DFBOLSPATH/libdfbols.a  
		
	# $DFBOLSFILE1 $DFBOLSFILE2 $DFBOLSFILE3 $DFBOLSFILE4 $DFBOLSFILE5 $DFBOLSFILE6 $DFBOLSFILE7 $DFLSPATH/libdfls.a \
	# Create debug file with all script information 
	# $DFBOLSPATH/libdfbols.a  $DFLSPATH/libdfls.a 
	# 	
	dsymutil $2.out


	# -g tells compiler to produce symbol info
	# -save-temps tells compiler not to delete intermediate files and puts them in working directory

	if [ "$3" == "run" ]; then
		source /opt/intel/Compiler/11.1/058/bin/ifortvars.sh intel64
		./$2.out
	fi

# Release version with openmp
elif [ "$1" == "Release" ]; then
		echo "Compiling in " $1 "mode..."
		
		ifort -m64 -O2 -openmp -traceback -Wl,-stack_size,0x10000000 \
			Parameters.f90 Globals.f90 Procedures.f90  OtherCode/random.f90 \
			SetParameters.f90 LoadData.f90 Grids.f90 Estimation.f90 MomentConditions.f90	\
			Decisions.f90 Simulate.f90 DrawRandomNumbers.f90 \
			CalculateStats.f90 SaveOutput.f90 \
			Functions/FnAggHours.f90 Functions/FnBetaKY.f90 Functions/FnHoursBC.f90 Functions/FnSSParam.f90 Functions/dfovec.f90 \
			Functions/FnZgrid.f90 Functions/FnGridFixedEffect.f90 Functions/FnGridPrefHet.f90 Functions/FnGridPrefHetTrans.f90 \
			Functions/FnGridTrans.f90 Functions/FnTax.f90 \
			OtherCode/rtsecBetaKY.f90 OtherCode/rtsecHoursBC.f90 OtherCode/rtbisHoursBC.f90 OtherCode/rtflspHoursBC.f90 OtherCode/rtsecSSParam.f90 OtherCode/rtsecAggHours.f90 \
			OtherCode/minpack.f90 OtherCode/golden.f90 OtherCode/sort.f90 OtherCode/cumnor.f90 OtherCode/mnbrak.f90 OtherCode/rtbis.f90 \
			$DFLSFILE1 $DFLSFILE2 $DFLSFILE3 $DFLSFILE4 $DFLSFILE5 $DFLSFILE6 \
			Main.f90 -o $2.out \
			$DFBOLSPATH/libdfbols.a  

			
	# -fast inclues-O3 plus some other optimization things
	# -openmp-link static forces the openmp libraries to be linked staticaly.
	# for some reason, when i try use the library version of DFLS i get an error that is related memory issues or segmentation faults. i think that it has to do with the way the DFLS library is compiled. presumably DFBOLS will hit this problem at some point too.
	# $DFLSFILE1 $DFLSFILE2 $DFLSFILE3 $DFLSFILE4 $DFLSFILE5 $DFLSFILE6 \
	# $DFBOLSFILE1 $DFBOLSFILE2 $DFBOLSFILE3 $DFBOLSFILE4 $DFBOLSFILE5 $DFBOLSFILE6 $DFBOLSFILE7 \
	# $DFLSPATH/libdfls.a  
	
	
	if [ "$3" == "run" ]; then
		source /opt/intel/Compiler/11.1/058/bin/ifortvars.sh intel64
		./$2.out
	fi


fi

