SUBROUTINE Decisions

USE Parameters
USE Globals
USE Procedures

IMPLICIT NONE

! Local Variables
integer ::  it,ia,ip,ie,iz,ie2,iz2,iff,if2,ifP,ik,BLbind,iazefh,ih,ih2,info
REAL(8)			::  emuc(ngpk),lminh,lmaxh
REAL(8), EXTERNAL        :: FnHoursBC,FnTax


!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!			
! Start with last period, eat everything  !!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!			

if (Display>=2) write(*,*) 'Solving for decision rules at age ', Ttot
gannprem = annprem(Ttot)

DO ip = 1,ngpp
	DO ik = 1,ngpk
	    conret(Tret,ik,ip) = ( Rnet*kgridret(Tret,ik,ip)/gannprem + pnetgrid(Tret,ip) )/ (1.0+ctax)
		mucret(Tret,ik,ip) = conret(Tret,ik,ip)**(-gam)			
	END DO
END DO


!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!! Retired Agents: Pension value as state variable !!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

DO it = Tret-1,1,-1
	if (Display>=2) write(*,*) 'Solving for decision rules at age ', Twork+it				
	gannprem = annprem(Twork+it)
	gannprem1 = annprem(Twork+it+1)
	gsurprob = surprob(Twork+it)
	
	!$OMP PARALLEL DO PRIVATE(BLbind) COPYIN(gannprem,gannprem1,gsurprob)
	DO ip = 1,ngpp
	
		!solve on tt+1 grid
		mucret1(it,:,ip) = bet*Rnet*(gsurprob/gannprem1)*mucret(it+1,:,ip)		
		conret1(it,:,ip) = mucret1(it,:,ip)**(-1.0/gam)  
		assret1(it,:,ip) = (conret1(it,:,ip)*(1.0+ctax) + kgridret(it+1,:,ip)-pnetgrid(it,ip) )*gannprem/Rnet

		!deal with borrowing limits
		IF (MINVAL(kgridret(it,:,ip)) >= assret1(it,1,ip)) THEN		!BL does not bind anywhere
			BLbind = 0
		ELSE				!find point in tt grid where BL starts to bind
			BLbind = MAXLOC(kgridret(it,:,ip), DIM=1,MASK = kgridret(it,:,ip) .lt. assret1(it,1,ip))
			assret(it,1:BLbind,ip)	= kgridret(it+1,1,ip)
			conret(it,1:BLbind,ip)	= (Rnet*kgridret(it,1:BLbind,ip)/gannprem + pnetgrid(it,ip) - assret(it,1:BLbind,ip)) / (1.0+ctax)
    		mucret(it,1:BLbind,ip)    = conret(it,1:BLbind,ip)**(-gam)
		END IF
		
		!interpolate muc1 as fun of ass1 to get con where BL does not bind
		CALL LinInterp (ngpk,assret1(it,:,ip),conret1(it,:,ip),&
						ngpk-BLbind,kgridret(it,BLbind+1:ngpk,ip),conret(it,BLbind+1:ngpk,ip))
        mucret(it,BLbind+1:ngpk,ip) = conret(it,BLbind+1:ngpk,ip)**(-gam)
		assret(it,BLbind+1:ngpk,ip)	= Rnet*kgridret(it,BLbind+1:ngpk,ip)/gannprem + pnetgrid(it,ip) - conret(it,BLbind+1:ngpa,ip)*(1.0+ctax)

		if (any(isnan(conret(it,:,ip)))) THEN
			write(*,*) 'nan encountered'
		end if

	END DO  !ip: pension grid
    !$OMP END PARALLEL DO

END DO	


!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!			
!! Working Agents: Rules depend on shocks  !!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

DO it = Twork,1,-1
	if (Display>=2) write(*,*) 'Solving for decision rules at age ', it				
	gannprem = annprem(it)
	gannprem1 = annprem(it+1)
	gsurprob = surprob(it)
    
	!$OMP PARALLEL DO PRIVATE(ia,iz,ie,iff,if2,ifP,ip,BLbind,emuc,iz2,ie2,ih2,ik,ih,lminh,lmaxh,info) COPYIN(gannprem,gannprem1,gsurprob)
	DO iazefh = 1,ngpa*ngpz*ngpe*ngpf*ngphbar
	    ia = aloopgrid(iazefh) 
	    iz = zloopgrid(iazefh) 
	    ie = eloopgrid(iazefh) 
	    iff = floopgrid(iazefh)
	 	ifP = fPind(iff)
	    ih  = hloopgrid(iazefh)

	    ip = pind(iz,ia)
	    gwage = wgrid(it,ia,iz,ie)	!this is gross
	    gpsi = fgrid(it,iff)
	    gben = ben*(1-hgrid(ih))  !this is gross
		ghbar = hgrid(ih)
		ghbaradj = ghbar**(chi*(1+sig))
		    
        !solve on tt+1 grid
		IF (it==Twork) THEN			
			muc1(it,:,ia,iz,ie,iff,ih) = bet*Rnet*(gsurprob/gannprem1)* mucret(1,:,pind(iz,ia))		!euler equation
        ELSE
			emuc(:) = 0.0
			DO ih2 = 1,ngphbar
			DO iz2 = 1,ngpz
			DO ie2 = 1,ngpe
			DO if2 = 1,ngpfT
				emuc = emuc + muc(it+1,:,ia,iz2,ie2,find(ifP,if2),ih2)*ztrans(it,iz,iz2)*edist(it+1,ie2)*hdist(it+1,ih2,ifP)*fTdist(it+1,if2)
			END DO
			END DO
			END DO
			END DO
			muc1(it,:,ia,iz,ie,iff,ih)	=  bet*(gsurprob/gannprem1)*Rnet*emuc 				!euler equation
		END IF
        con1(it,:,ia,iz,ie,iff,ih) = muc1(it,:,ia,iz,ie,iff,ih)**(-1.0/gam) 

		if (any(isnan(con1(it,:,ia,iz,ie,iff,ih)))) THEN
			write(*,*) 'nan encountered'
		end if
		
		!Without taxes, hours are closed form: hour is hours per week.
		mul1(it,:,ia,iz,ie,iff,ih) = gwage*ghbar*muc1(it,:,ia,iz,ie,iff,ih) / (1.0+ctax)
		if(LeisUtilPower==1) hour1(it,:,ia,iz,ie,iff,ih) = 1.0 - (mul1(it,:,ia,iz,ie,iff,ih)/gpsi)**(-1.0/sig)
		if(LeisUtilExpon==1) hour1(it,:,ia,iz,ie,iff,ih) = 1.0 - (1.0/sig) * log(mul1(it,:,ia,iz,ie,iff,ih)/gpsi) !Note old mistake
		if(HourUtilPower==1) hour1(it,:,ia,iz,ie,iff,ih) = (mul1(it,:,ia,iz,ie,iff,ih)*((gwage*ghbar)**ltax2)*ltax1/(ghbaradj*gpsi))**(1.0/(sig-ltax2))
		if(HourUtilExpon==1) hour1(it,:,ia,iz,ie,iff,ih) = (1.0/sig) * log(mul1(it,:,ia,iz,ie,iff,ih)/gpsi) 
        hour1(it,:,ia,iz,ie,iff,ih) = min(max(hour1(it,:,ia,iz,ie,iff,ih),0.0),1.0)
		
		    
		DO ik = 1,ngpk
	        IF (it==Twork) ass1(it,ik,ia,iz,ie,iff,ih) = (con1(it,ik,ia,iz,ie,iff,ih)*(1.0+ctax) +kgridret(1,ik,ip) - hour1(it,ik,ia,iz,ie,iff,ih)*ghbar*gwage - gben + FnTax(hour1(it,ik,ia,iz,ie,iff,ih)*ghbar*gwage + gben) )*gannprem/Rnet  
	        IF (it<Twork) ass1(it,ik,ia,iz,ie,iff,ih) = (con1(it,ik,ia,iz,ie,iff,ih)*(1.0+ctax) +kgrid(it+1,ik,ia) - hour1(it,ik,ia,iz,ie,iff,ih)*ghbar*gwage - gben + FnTax(hour1(it,ik,ia,iz,ie,iff,ih)*ghbar*gwage+gben) )*gannprem/Rnet  
		END DO
		
        !deal with borrowing limits
        IF (MINVAL(kgrid(it,:,ia)) >= ass1(it,1,ia,iz,ie,iff,ih) + 1.0e-6) THEN		!BL does not bind anywhere
	        BLbind = 0	
        ELSE
            !find point in tt grid where BL starts to bind
	        BLbind = MAXLOC(kgrid(it,:,ia), DIM=1,MASK = kgrid(it,:,ia) .lt. ass1(it,1,ia,iz,ie,iff,ih) + 1.0e-5)						
		    IF (it==Twork) THEN	
			    ass(it,1:BLbind,ia,iz,ie,iff,ih) = kgridret(1,1,ip)
		    ELSE
			    ass(it,1:BLbind,ia,iz,ie,iff,ih) = kgrid(it+1,1,ia)
		    END IF
    		
    		!find optimal hours using secant method
            DO ik = 1,BLbind
                gass = kgrid(it,ik,ia)
                gbc = ass(it,ik,ia,iz,ie,iff,ih)
                IF (FnHoursBC(0.0_8)>0.0 .and. HourUtilPower .ne. 1) THEN 	!no corner at zero for hours util power
                    hour(it,ik,ia,iz,ie,iff,ih) = 0.0_8
                    con(it,ik,ia,iz,ie,iff,ih)  = (Rnet*gass/gannprem - gbc + gben)/(1.0+ctax)
                    muc(it,ik,ia,iz,ie,iff,ih)  = con(it,ik,ia,iz,ie,iff,ih)**(-gam)
                    if(LeisUtilPower==1) mul(it,ik,ia,iz,ie,iff,ih) = gpsi
                    if(LeisUtilExpon==1) mul(it,ik,ia,iz,ie,iff,ih) = gpsi*exp(sig)
                    if(HourUtilExpon==1) mul(it,ik,ia,iz,ie,iff,ih) = gpsi

                ELSE IF (FnHoursBC(1.0_8)<0.0 .and. LeisUtilPower .ne. 1) THEN  !no corner at zero for leis util power
                    hour(it,ik,ia,iz,ie,iff,ih) = 1.0
                    con(it,ik,ia,iz,ie,iff,ih) = (Rnet*gass/gannprem - gbc + ghbar*gwage+ gben -FnTax(ghbar*gwage+gben) )/(1.0+ctax)
                    muc(it,ik,ia,iz,ie,iff,ih) = con(it,ik,ia,iz,ie,iff,ih)**(-gam)
                    if(LeisUtilExpon==1) mul(it,ik,ia,iz,ie,iff,ih) = gpsi
                    if(HourUtilPower==1) mul(it,ik,ia,iz,ie,iff,ih) = gpsi*ghbaradj/(ltax1*((gwage*ghbar)**ltax2))
                    if(HourUtilExpon==1) mul(it,ik,ia,iz,ie,iff,ih) = gpsi*exp(sig)

                ELSE
					lminh = 0.0
					lmaxh = 1.0
                    if(HourUtilPower==1) lminh = 1.0e-6
                    if(LeisUtilPower==1) lmaxh = 1.0 -1.0e-6

                    !CALL rtflspHoursBC(lminh,lmaxh,1.0e-4_8,hour(it,ik,ia,iz,ie,iff,ih),info) 
                    CALL rtsecHoursBC(hour1(it,1,ia,iz,ie,iff,ih),min(hour1(it,1,ia,iz,ie,iff,ih)*1.1,1.0),1.0e-4_8,hour(it,ik,ia,iz,ie,iff,ih),info) 
					IF(info<0) CALL rtbisHoursBC(lminh,lmaxh,1.0e-7_8,hour(it,ik,ia,iz,ie,iff,ih),info) 
                    con(it,ik,ia,iz,ie,iff,ih) = (Rnet*gass/gannprem + ghbar*gwage*hour(it,ik,ia,iz,ie,iff,ih)+ gben -FnTax(ghbar*gwage*hour(it,ik,ia,iz,ie,iff,ih)+gben) - gbc)/(1.0+ctax)
                    if(con(it,ik,ia,iz,ie,iff,ih) < 0) then
						IF(Display>=2) write(*,*) 'problem with HoursBC rootfinding'
                        con(it,ik,ia,iz,ie,iff,ih) = cmin
                        hour(it,ik,ia,iz,ie,iff,ih) = ( ((con(it,ik,ia,iz,ie,iff,ih)*(1.0+ctax) - Rnet*gass/gannprem - gben + gbc +ltax0)*(ltax2+1.0)/ltax1)**(1.0/(1.0+ltax2)) )/ (gwage*ghbar) 
                    end if                     
                    muc(it,ik,ia,iz,ie,iff,ih) = con(it,ik,ia,iz,ie,iff,ih)**(-gam)
                    if(LeisUtilPower==1) mul(it,ik,ia,iz,ie,iff,ih) = gpsi*(1.0-hour(it,ik,ia,iz,ie,iff,ih))**(-sig)
                    if(LeisUtilExpon==1) mul(it,ik,ia,iz,ie,iff,ih) = gpsi*dexp(sig*(1.0-hour(it,ik,ia,iz,ie,iff,ih))) !mistake corrected
                    if(HourUtilPower==1) mul(it,ik,ia,iz,ie,iff,ih) = ghbaradj*gpsi*(hour(it,ik,ia,iz,ie,iff,ih)**(sig-ltax2)) / (ltax1*((gwage*ghbar)**ltax2))
                    if(HourUtilExpon==1) mul(it,ik,ia,iz,ie,iff,ih) = gpsi*dexp(sig*(hour(it,ik,ia,iz,ie,iff,ih)))
                    
                END IF                    
            END DO		    
        END IF

    					
	    !interpolate con1 as fun of ass1 to get muc where BL does not bind		
	    CALL LinInterp (ngpk,ass1(it,:,ia,iz,ie,iff,ih),con1(it,:,ia,iz,ie,iff,ih),&
					    ngpk-BLbind,kgrid(it,BLbind+1:ngpk,ia),con(it,BLbind+1:ngpk,ia,iz,ie,iff,ih) )
        muc(it,BLbind+1:ngpk,ia,iz,ie,iff,ih) = con(it,BLbind+1:ngpk,ia,iz,ie,iff,ih)**(-gam)
        
		mul(it,BLbind+1:ngpk,ia,iz,ie,iff,ih) = gwage*ghbar*muc(it,BLbind+1:ngpk,ia,iz,ie,iff,ih)/(1.0+ctax)
		if(LeisUtilPower==1) hour(it,BLbind+1:ngpk,ia,iz,ie,iff,ih) = 1.0 - (mul(it,BLbind+1:ngpk,ia,iz,ie,iff,ih)/gpsi)**(-1.0/sig)
		if(LeisUtilExpon==1) hour(it,BLbind+1:ngpk,ia,iz,ie,iff,ih) = 1.0 - (1.0/sig) * dlog(mul(it,BLbind+1:ngpk,ia,iz,ie,iff,ih)/gpsi)	!mistake corrected	
		if(HourUtilPower==1) hour(it,BLbind+1:ngpk,ia,iz,ie,iff,ih) = (mul(it,BLbind+1:ngpk,ia,iz,ie,iff,ih)*ltax1*((gwage*ghbar)**ltax2)/(gpsi*ghbaradj))**(1.0/(sig-ltax2))
		if(HourUtilExpon==1) hour(it,BLbind+1:ngpk,ia,iz,ie,iff,ih) = (1.0/sig) * dlog(mul(it,BLbind+1:ngpk,ia,iz,ie,iff,ih)/gpsi)		
		hour(it,BLbind+1:ngpk,ia,iz,ie,iff,ih) = min(max(hour(it,BLbind+1:ngpk,ia,iz,ie,iff,ih),0.0),1.0)

		DO ik=BLbind+1,ngpk
        	ass(it,ik,ia,iz,ie,iff,ih) = Rnet*kgrid(it,ik,ia)/gannprem + ghbar*gwage*hour(it,ik,ia,iz,ie,iff,ih) + gben -FnTax(ghbar*gwage*hour(it,ik,ia,iz,ie,iff,ih)+gben) - con(it,ik,ia,iz,ie,iff,ih)*(1.0+ctax)
		END DO
		
		if (any(isnan(con(it,:,ia,iz,ie,iff,ih)))) THEN
			write(*,*) 'nan encountered'
		end if

        !some times borrowing limits may be violated by a small amount because of inaccuracies in interpolation.
		!this assumes that at the problem point, the BC binds. need to check whether there is a better way
		DO ik = BLbind+1,min(ngpk,BLbind+2)	 !check two points up
			IF (it<Twork) gbc=kgrid(it+1,1,ia)
			IF (it==Twork) gbc=kgridret(1,1,ip)
            IF(ass(it,ik,ia,iz,ie,iff,ih)<gbc ) THEN
				ass(it,ik,ia,iz,ie,iff,ih) = gbc
                gass = kgrid(it,ik,ia)
				lminh = 0.0
				lmaxh = 1.0
                if(HourUtilPower==1) lminh = 1.0e-6
                if(LeisUtilPower==1) lmaxh = 1.0 -1.0e-6					
            
    			!CALL rtflspHoursBC(lminh,lmaxh,1.0e-4_8,hour(it,ik,ia,iz,ie,iff,ih),info) 
				CALL rtsecHoursBC(hour1(it,1,ia,iz,ie,iff,ih),min(hour1(it,1,ia,iz,ie,iff,ih)*1.1,1.0),1.0e-4_8,hour(it,ik,ia,iz,ie,iff,ih),info) 
				IF(info<0) CALL rtbisHoursBC(lminh,lmaxh,1.0e-7_8,hour(it,ik,ia,iz,ie,iff,ih),info) 
				
				con(it,ik,ia,iz,ie,iff,ih) = (Rnet*gass/gannprem + ghbar*gwage*hour(it,ik,ia,iz,ie,iff,ih)+ gben -FnTax(ghbar*gwage*hour(it,ik,ia,iz,ie,iff,ih) + gben)  - gbc)/(1.0+ctax)
                if(con(it,ik,ia,iz,ie,iff,ih) < 0) then
					IF(Display>=2) write(*,*) 'problem with HoursBC rootfinding'
					con(it,ik,ia,iz,ie,iff,ih) = cmin
					hour(it,ik,ia,iz,ie,iff,ih) = ( ((con(it,ik,ia,iz,ie,iff,ih)*(1.0+ctax) - Rnet*gass/gannprem - gben + gbc +ltax0)*(ltax2+1.0)/ltax1)**(1.0/(1.0+ltax2)) )/ (gwage*ghbar)
				end if
				muc(it,ik,ia,iz,ie,iff,ih) = con(it,ik,ia,iz,ie,iff,ih)**(-gam)
		    	mul(it,ik,ia,iz,ie,iff,ih) = gwage*ghbar*muc(it,ik,ia,iz,ie,iff,ih)/(1.0+ctax)
			END IF

		END DO !ik, borrowing constraint
		
    END DO  !azefhgrid
    !$OMP END PARALLEL DO

! write(*,*) sum(hour(it,:,:,:,:,:,:))
! write(*,*) sum(hour1(it,:,:,:,:,:,:))

END DO	!t

!write(*,*) maxval(hour1)

END SUBROUTINE Decisions
