SUBROUTINE Estimation

USE Parameters
USE Globals
USE Procedures

IMPLICIT NONE

INTEGER					:: ip,npt,maxfun,iprint,j
REAL(8)					:: rhobeg, rhoend
REAL(8), ALLOCATABLE	:: x(:),w(:),xl(:),xu(:)

!set up parameters
nparam = 0
IF (EstimateGamma == 1) 		nparam = nparam+1
IF (EstimateSigma == 1) 		nparam = nparam+1
IF (EstimateMeanPref == 1) 		nparam = nparam+1
IF (EstimateVarPref == 1) 		nparam = nparam+1
IF (EstimateVarPrefTrans == 1) 	nparam = nparam+1
IF (EstimateRhoPref == 1) 		nparam = nparam+1
IF (EstimateBeta == 1) 			nparam = nparam+1
IF (EstimateChi == 1) 			nparam = nparam+1
IF (EstimateBorrowLim == 1)		nparam = nparam+1
IF (EstimateErrC == 1)			nparam = nparam+1
IF (EstimateErrY == 1)			nparam = nparam+1
IF (EstimateErrH == 1)			nparam = nparam+1

IF (ALLOCATED(paramguess)) DEALLOCATE(paramguess)
IF (ALLOCATED(paramout)) DEALLOCATE(paramout)
IF (ALLOCATED(paramscale)) DEALLOCATE(paramscale)
IF (ALLOCATED(paramub)) DEALLOCATE(paramub)
IF (ALLOCATED(paramlb)) DEALLOCATE(paramlb)
IF (ALLOCATED(x)) DEALLOCATE(x)
IF (ALLOCATED(xl)) DEALLOCATE(xl)
IF (ALLOCATED(xu)) DEALLOCATE(xu)
 ALLOCATE(paramguess(nparam),paramout(nparam),paramscale(nparam),paramub(nparam),paramlb(nparam),x(nparam),xl(nparam),xu(nparam))

!set up moments
nmoments = 0
IF (MatchMeanCon == 1)		nmoments = nmoments+1
IF (MatchMeanHour == 1)		nmoments = nmoments+1
IF (MatchVarCon == 1)		nmoments = nmoments+1
IF (MatchVarHour == 1)		nmoments = nmoments+1
IF (MatchCovWageHour == 1)	nmoments = nmoments+1
IF (MatchCovConWage == 1)	nmoments = nmoments+1
IF (MatchCovConHour == 1)	nmoments = nmoments+1
IF (MatchZeroNegWealth == 1) nmoments = nmoments+1
nmoments = nmoments*36	!assumes match 3-38 exper, Twork=1-36 for all moments.

!set up weighting matrix and n vector
IF (ALLOCATED(diagweight)) DEALLOCATE(diagweight)
IF (ALLOCATED(nobsvec)) DEALLOCATE(nobsvec)
ALLOCATE(diagweight(nmoments),nobsvec(nmoments))
diagweight = 1.0

!make guess: with bound constraints, scale is set in one go below, based on bounds. the initial starting points are the midpoints of the bounds
ip = 0
IF(EstimateGamma==1) THEN
    ip=ip+1
	IF(UseBoundConstraints==1) THEN
		paramlb(ip) = 0.5
		paramub(ip) = 6.0
		paramguess(ip) = gamG !2.0*(gamG - paramlb(ip))/(paramub(ip)-paramlb(ip))-1.0
	ELSE
	    paramguess(ip) = log(gamG)
	    paramlb(ip) = 0.1
	    paramub(ip) = 6.0
	    paramscale(ip) = 1.0/(log(paramub(ip))-paramguess(ip))
	END IF
ELSE
    gam = gamG
END IF

IF(EstimateSigma==1) THEN
    ip=ip+1
	IF(UseBoundConstraints==1) THEN
		paramlb(ip) = 1.01
		paramub(ip) = 15.0
		paramguess(ip) = sigG !2.0*(sigG - paramlb(ip))/(paramub(ip)-paramlb(ip))-1.0
	ELSE
	    paramguess(ip) = log(sigG-1.0)	!sig>1
	    paramlb(ip) = 1.1
	    paramub(ip) = 10.0
	    paramscale(ip) = 1.0/(log(paramub(ip)-1.0)-paramguess(ip))
	END IF
ELSE
    sig = sigG
END IF

IF(EstimateMeanPref==1) THEN
    ip=ip+1
	IF(UseBoundConstraints==1) THEN
		paramlb(ip) = 1.0e-8
		paramub(ip) = 1000.0
		paramguess(ip) = EpsipowG
	ELSE
	    paramguess(ip) = log(EpsipowG)	
	    paramlb(ip) = 0.001
	    paramub(ip) = 3.0*EpsipowG
	    paramscale(ip) = 1.0/(log(paramub(ip))-paramguess(ip))
	END IF
ELSE
	Epsipow = EpsipowG
END IF


IF(EstimateVarPref==1) THEN
    ip=ip+1
	IF(UseBoundConstraints==1) THEN
		paramlb(ip) = 0.0
		paramub(ip) = 2.0
		paramguess(ip) = CVpsipowG
	ELSE
	    paramguess(ip) = log(CVpsipowG)
	    paramlb(ip) = 0.0
	    paramub(ip) = 2.0
	    paramscale(ip) = 1.0/(log(paramub(ip))-paramguess(ip))
	END IF
ELSE
	CVpsipow = CVpsipowG
END IF


IF(EstimateVarPrefTrans==1) THEN
    ip=ip+1
	IF(UseBoundConstraints==1) THEN
		paramlb(ip) = 0.0
		paramub(ip) = 0.2
		paramguess(ip) = CVzetapowG
	ELSE
	    paramguess(ip) = log(CVzetapowG)
	    paramlb(ip) = 0.0
	    paramub(ip) = 1.0
	    paramscale(ip) = 1.0/(log(paramub(ip))-paramguess(ip))
	END IF
ELSE
	CVzetapow = CVzetapowG
END IF

IF(EstimateRhoPref==1) THEN
    ip=ip+1
	IF(UseBoundConstraints==1) THEN
		paramlb(ip) = -1.0
		paramub(ip) = 2.0
		paramguess(ip) = rhoprefG
	ELSE
	    paramguess(ip) = rhoprefG
	    paramlb(ip) = -1.0
	    paramub(ip) = 1.0
	    paramscale(ip) = 1.0/(paramub(ip)-paramguess(ip))
	END IF
ELSE
	rhopref = rhoprefG
END IF

IF(EstimateBeta==1) THEN
    ip=ip+1
	IF(UseBoundConstraints==1) THEN
		paramlb(ip) = 0.9
		paramub(ip) = 1.1
		paramguess(ip) = betG
	ELSE
	    paramguess(ip) = invlogistic(betG)	!
	    paramlb(ip) = 0.95
	    paramub(ip) = 1.0
	    paramscale(ip) = -1.0/(invlogistic(paramlb(ip))-paramguess(ip))
	END IF
ELSE
    bet = betG
END IF

IF(EstimateChi==1) THEN
    ip=ip+1
	IF(UseBoundConstraints==1) THEN
		paramlb(ip) = -1.999
		paramub(ip) = 0.999
		paramguess(ip) = chiG
	ELSE
! 	    paramguess(ip) = log(1.0-chiG)
! 	    paramlb(ip) = -1.999
! 	    paramub(ip) = 0.999
! 	    paramscale(ip) = 1.0/(log(1.0-paramlb(ip))-paramguess(ip))

	    paramguess(ip) = log(chitildG)
	    paramlb(ip) = 0.001
	    paramub(ip) = 6.0
	    paramscale(ip) = 1.0/(log(paramub(ip))-paramguess(ip))
	END IF
ELSE
    chi = chiG
	chitild = chitildG
END IF

IF(EstimateBorrowLim==1) THEN
    ip=ip+1
	IF(UseBoundConstraints==1) THEN
		paramlb(ip) = -10.0
		paramub(ip) = 0.0
		paramguess(ip) = borrowlimG
	ELSE
	    paramguess(ip) = log(-borrowlimG)	!borrowlim<=0.0
	    paramlb(ip) = -10.0
	    paramub(ip) = -0.001
	    paramscale(ip) = 1.0/(log(-paramlb(ip) +paramguess(ip)))
	END IF
ELSE
    borrowlim = borrowlimG
END IF

IF(EstimateErrC==1) THEN
    ip=ip+1
	IF(UseBoundConstraints==1) THEN
		paramlb(ip) = 0.0
		paramub(ip) = 0.5
		paramguess(ip) = err_cG
	ELSE
	    paramguess(ip) = log(err_cG)	
	    paramlb(ip) = 0.0001
	    paramub(ip) = 0.5
	    paramscale(ip) = 1.0/(log(paramub(ip))-paramguess(ip))
	END IF
ELSE
    err_c = err_cG
END IF

IF(EstimateErrY==1) THEN
    ip=ip+1
	IF(UseBoundConstraints==1) THEN
		paramlb(ip) = 0.0
		paramub(ip) = 0.5
		paramguess(ip) = err_yG
	ELSE
	    paramguess(ip) = log(err_yG)	
	    paramlb(ip) = 0.0001
	    paramub(ip) = 0.5
	    paramscale(ip) = 1.0/(log(paramub(ip))-paramguess(ip))
	END IF
ELSE
    err_y = err_yG
END IF

IF(EstimateErrH==1) THEN
    ip=ip+1
	IF(UseBoundConstraints==1) THEN
		paramlb(ip) = 0.0
		paramub(ip) = 0.5
		paramguess(ip) = err_hG
	ELSE
	    paramguess(ip) = log(err_hG)	
	    paramlb(ip) = 0.0001
	    paramub(ip) = 0.5
	    paramscale(ip) = 1.0/(log(paramub(ip))-paramguess(ip))
	END IF
ELSE
    err_h = err_hG
END IF

!write paramguess to file
OPEN(3, FILE = trim(OutputBaseDir) // 'paramguess' //   '.txt', STATUS = 'replace')
CALL WriteMatrix(3,Nparam,1,paramguess)
OPEN(3, FILE = trim(OutputBaseDir) // 'paramlb' //   '.txt', STATUS = 'replace')
CALL WriteMatrix(3,Nparam,1,paramlb)
OPEN(3, FILE = trim(OutputBaseDir) // 'paramub' //   '.txt', STATUS = 'replace')
CALL WriteMatrix(3,Nparam,1,paramub)

IF(UseBoundConstraints==1) paramscale = 2.1/(paramub-paramlb)


ndfls = 1
x = paramguess*paramscale
xl = paramlb*paramscale
xu = paramub*paramscale
npt = 2*nparam+1
rhobeg = 1.0
rhoend = 1.0D-5	!1.0D-8
maxfun = 500*(nparam+1) 
iprint = 3
IF(ALLOCATED(w)) DEALLOCATE(w)
ALLOCATE(w((npt+11)*(npt+nparam) +nparam*(3*nparam+11)/2) )

!do DFLS/DFBOLS minimization: objective function in dfovec.f90
!NB: DFBOLS automatically makes points the midpoints of the bounds
!DFLS uses the actual guesses with no bounds
OPEN(4, FILE = trim(OutputDir) // 'iterations' //   '.txt', STATUS = 'replace')
DO j = 1,ndfls
	write(4,*) '********************************** '
	write(4,*) 'DFLS/DFBOLS MINIMIZATION ATTEMPT ', j
	IF(UseBoundConstraints==0) CALL NEWUOA_H(nparam,npt,x,rhobeg,rhoend,iprint,maxfun,w,nmoments)
	IF(UseBoundConstraints==1) CALL BOBYQA_H(nparam,npt,x,xl,xu,rhobeg,rhoend,iprint,maxfun,w,nmoments)
	write(4,*) '********************************** '
END DO
CLOSE(4)
paramout = x(1:nparam)/paramscale


!extract parameters at solution
ip = 0
IF(EstimateGamma==1) THEN
    ip=ip+1
    IF (UseBoundConstraints==1)	gam =paramout(ip)
    IF (UseBoundConstraints==0)    gam = exp(paramout(ip))
	write(*,*) ' gam sol: ',gam
END IF

IF(EstimateSigma==1) THEN
    ip=ip+1
    IF (UseBoundConstraints==1)	sig = paramout(ip) 
    IF (UseBoundConstraints==0)    sig = exp(paramout(ip)) + 1.0
	write(*,*) ' sig sol: ',sig
END IF

IF(EstimateMeanPref==1) THEN
    ip=ip+1
    IF (UseBoundConstraints==1)Epsipow = paramout(ip)
    IF (UseBoundConstraints==0)	Epsipow = exp(paramout(ip))
	write(*,*) ' Epsipow sol: ',Epsipow
END IF

IF(EstimateVarPref==1) THEN
    ip=ip+1
    IF (UseBoundConstraints==1)CVpsipow = paramout(ip)
    IF (UseBoundConstraints==0)	CVpsipow = exp(paramout(ip))
	write(*,*) ' CVpsipow sol: ',CVpsipow
END IF

IF(EstimateVarPrefTrans==1) THEN
    ip=ip+1
    IF (UseBoundConstraints==1)CVzetapow = paramout(ip)
    IF (UseBoundConstraints==0)	CVzetapow = exp(paramout(ip))
	write(*,*) ' CVzetapow sol: ',CVzetapow
END IF

IF(EstimateRhoPref==1) THEN
    ip=ip+1
    IF (UseBoundConstraints==1)rhopref = paramout(ip)
    IF (UseBoundConstraints==0)	rhopref = paramout(ip)
	write(*,*) ' rhopref sol: ',rhopref
END IF

IF(EstimateBeta==1) THEN
    ip=ip+1
    IF (UseBoundConstraints==1)bet = paramout(ip)
    IF (UseBoundConstraints==0)    bet = logistic(paramout(ip))
	write(*,*) ' bet sol: ',bet
END IF

IF(EstimateChi==1) THEN
    ip=ip+1
    IF (UseBoundConstraints==1)chi = paramout(ip)
!     IF (UseBoundConstraints==0)	chi = 1.0-exp(paramout(ip))
    IF (UseBoundConstraints==0)	THEN
		chitild = exp(paramout(ip))
		chi = 1.0-chitild/(1.0+sig)
	END IF
	write(*,*) ' chitild sol: ',chitild
	write(*,*) ' chi sol: ',chi
END IF

IF(EstimateBorrowLim==1) THEN
    ip=ip+1
    IF (UseBoundConstraints==1)borrowlim = paramout(ip)
    IF (UseBoundConstraints==0)	borrowlim = -exp(paramout(ip))
	write(*,*) ' borrowlim sol: ',borrowlim
END IF

IF(EstimateErrC==1) THEN
    ip=ip+1
    IF (UseBoundConstraints==1)err_c = paramout(ip)
    IF (UseBoundConstraints==0)	err_c = exp(paramout(ip))
	write(*,*) ' err c sol: ',err_c
END IF

IF(EstimateErrY==1) THEN
    ip=ip+1
    IF (UseBoundConstraints==1)err_y = paramout(ip)
    IF (UseBoundConstraints==0)	err_y = exp(paramout(ip))
	write(*,*) ' err y sol: ',err_y
END IF

IF(EstimateErrH==1) THEN
    ip=ip+1
    IF (UseBoundConstraints==1)err_h = paramout(ip)
    IF (UseBoundConstraints==0)	err_h = exp(paramout(ip))
	write(*,*) ' err h sol: ',err_h
END IF


IF(EstimateErrH==1 .and. EstimateErrY==1) THEN
	err_w = err_h + err_y
END IF

Vpsipow = (Epsipow*CVpsipow)**2.0
Vlogpsipow = log(Vpsipow/(Epsipow**2.0) +1.0)
Elogpsipow = log(Epsipow) - Vlogpsipow/2.0
Elogpsi = (1.0-gam)*Elogpsipow - (gam+sig)*log(targethours)
Epsi = exp((1.0-gam)*Elogpsipow +((1.0-gam)**2.0)*Vlogpsipow/2.0) / (targethours**(sig+gam))
Vlogpsi = ((1.0-gam)**2.0)*Vlogpsipow
Vlogzeta = (1.0-gamG**2.0)*log(CVzetapow**2.0+1.0)


END SUBROUTINE