SUBROUTINE FnZGrid(ln,lx,lfvec,IFLAG)

USE Parameters
USE Globals
USE Procedures

IMPLICIT NONE

INTEGER, INTENT(IN)     :: ln,IFLAG
REAL(8), INTENT(IN)  :: lx(ln)
REAL(8), INTENT(OUT) :: lfvec(ln)

REAL(8),DIMENSION(Twork)    :: lwidth,lvar
REAL(8),DIMENSION(Twork-1)    :: lrho,lcov
REAL(8)                     :: ltemp1,ltemp2,ltemp3,ltemp4,lconsig
INTEGER             :: iz1,iz2,it

IF (lx(2)<0.0) THEN
    lfvec(1) = -10000.0
    lfvec(2) = -10000.0
    return
END IF


! get boundaries and fill in with equally spaced points
DO it = 1,Twork
    zgrid(it,1)    = -lx(1)*sqrt(varz(it))
    zgrid(it,ngpz) = lx(1)*sqrt(varz(it))
    lwidth(it) = (zgrid(it,ngpz)-zgrid(it,1))/real(ngpz-1)
    DO iz1 = 2, ngpz-1
        zgrid(it,iz1) = zgrid(it,1) + lwidth(it)*(iz1-1)
    END DO        
END DO

! fill in transition matrix using normal distribution
DO it = 1,Twork-1
    lconsig = lx(2)
    DO iz1 = 1,ngpz
        CALL cumnor ((zgrid(it+1,1)+0.5*lwidth(it+1)-rho*zgrid(it,iz1))/lconsig, ztrans(it,iz1,1), ltemp2)
	    DO iz2 = 2,ngpz-1
	        CALL cumnor ((zgrid(it+1,iz2)+0.5*lwidth(it+1)-rho*zgrid(it,iz1))/lconsig, ltemp1, ltemp2)
	        CALL cumnor ((zgrid(it+1,iz2)-0.5*lwidth(it+1)-rho*zgrid(it,iz1))/lconsig, ltemp3, ltemp4)
            ztrans(it,iz1,iz2) = ltemp1 - ltemp3
        END DO
        CALL cumnor ((zgrid(it+1,ngpz)-0.5*lwidth(it+1)-rho*zgrid(it,iz1))/lconsig, ltemp3,ztrans(it,iz1,ngpz))
        ztrans(it,iz1,:) = ztrans(it,iz1,:)/sum(ztrans(it,iz1,:))
    END DO
END DO

!find distribution at first period
CALL cumnor( (zgrid(1,1)+0.5*lwidth(1))/sqrt(varz(1)), zdist(1,1), ltemp2)	
DO iz1 = 2,ngpz-1
    CALL cumnor ((zgrid(1,iz1)+0.5*lwidth(1))/sqrt(varz(1)), ltemp1, ltemp2)
    CALL cumnor ((zgrid(1,iz1)-0.5*lwidth(1))/sqrt(varz(1)), ltemp3, ltemp4)
    zdist(1,iz1) = ltemp1 - ltemp3
END DO
CALL cumnor ((zgrid(1,ngpz)-0.5*lwidth(1))/sqrt(varz(1)), ltemp3, zdist(1,ngpz))
zdist(1,:) = zdist(1,:)/sum(zdist(1,:))

!find unconditional distributions
DO it = 2,Twork
    zdist(it,:) = MATMUL(zdist(it-1,:),ztrans(it-1,:,:))
    zdist(it,:) = zdist(it,:) /sum(zdist(it,:) )
END DO


!find variance
DO it = 1,Twork
    lvar(it) = DOT_PRODUCT(zgrid(it,:)**2,zdist(it,:)) - DOT_PRODUCT(zgrid(it,:),zdist(it,:))**2
END DO

!find autocovariance
DO it = 1,Twork-1
    lcov(it) = 0.0
    DO iz1 = 1,ngpz
        DO iz2=1,ngpz
            lcov(it) = lcov(it) + zgrid(it,iz1)*zgrid(it+1,iz2)*zdist(it,iz1)*ztrans(it,iz1,iz2)
        END DO
    END DO
    lcov(it) = lcov(it) - DOT_PRODUCT(zgrid(it,:),zdist(it,:))*DOT_PRODUCT(zgrid(it+1,:),zdist(it+1,:))
    lrho(it) = lcov(it)/lvar(it)
END DO

!moments: average variance, average autocorrelation
lfvec(1)    = DOT_PRODUCT(lvar,popsize(1:Twork))/SUM(popsize(1:Twork)) - SUM(varz)/real(Twork)
lfvec(2)    = DOT_PRODUCT(lrho,popsize(2:Twork))/SUM(popsize(2:Twork)) - rho
!lfvec(1) = SUM(lvar)/real(Twork) - SUM(varz)/real(Twork)
!lfvec(2) = SUM(lrho)/real(Twork-1) - rho

!put values in globals
varzapprox = lvar
rhozapprox = lrho

END SUBROUTINE FnZGrid