MODULE Globals
USE Parameters

IMPLICIT NONE

!GLOBALS FOR DIRECTORIES
character(len=300)  OutputBaseDir
character(len=300)  DataDir,DataBaseDir
character(len=300)  InputWageEstDir,InputWageEstBaseDir
character(len=300)  InputParamFile
character(len=300)  OutputDir
character(len=300)  OutputDirSims

!GLOBALS FOR GRIDS

real(8), dimension(Twork,ngpe)              :: egrid        !transitory component
real(8), dimension(Twork,ngpz)              :: zgrid        !persistent component
real(8), dimension(ngpa)                    :: agrid        !fixed effect
real(8), dimension(Twork,ngpf)              :: fgrid        !pref het
real(8), dimension(Twork,ngpfT)             :: fTgrid        !pref het: transitory
real(8), dimension(ngpfP)             		:: fPgrid        !pref het: permanent
real(8), dimension(ngphbar)		            :: hgrid        !hours constraint

real(8), dimension(Twork,ngpk,ngpa)         :: kgrid        !assets
real(8), dimension(Twork,ngpa,ngpz,ngpe)    :: wgrid        !wages

real(8), dimension(Tret,ngpp)               :: pgrid,pnetgrid        !pension grid
real(8), dimension(Tret,ngpk,ngpp)          :: kgridret     !assets

real(8), dimension(Twork,ngpa)	            :: nbl		    !natural borrowing limits
real(8), dimension(Tret,ngpp)	            :: nblret	    !natural borrowing limits

integer, dimension(ngpa*ngpz*ngpe*ngpf*ngphbar) :: aloopgrid,zloopgrid,eloopgrid,floopgrid,hloopgrid
integer, dimension(ngpf)					:: fPind,fTind
integer, dimension(ngpfP,ngpfT)				:: find

!GLOBALS TO STORE DECISION RULES 
real(8), dimension(Twork,ngpk,ngpa,ngpz,ngpe,ngpf,ngphbar) :: con,ass,ass1,con1,muc,muc1,emuc1,val,hour1,hour,mul,mul1
real(8), dimension(Tret,ngpk,ngpp)	        :: conret,assret,conret1,assret1,mucret,mucret1,Emucret1,valret

!GLOBALS FOR EARNINGS DISTRIBUTIONS AND TRANSITION MAPRICES
real(8), dimension(Twork-1,ngpz,ngpz)	:: ztrans		
real(8), dimension(Twork,ngpz)			:: zdist
real(8), dimension(Twork)			    :: varz,varzapprox		
real(8), dimension(Twork-1)			    :: rhozapprox		
real(8), dimension(Twork,ngpe)		    :: edist	
real(8), dimension(ngpa)		        :: adist
real(8), dimension(ngpfP)		        :: fPdist	
real(8), dimension(Twork,ngpfT)		    :: fTdist	
real(8), dimension(Twork,ngpf)		    :: fdist	
real(8), dimension(Twork,ngphbar,ngpfP)		:: hdist	
real(8), dimension(Twork)		        :: kappa,avwage

!GLOBALS FOR PENSION SYSTEM
integer, dimension(ngpz,ngpa)	:: pind
real(8)     :: penrate

!PARAMETER GLOBALS
real(8)     :: vareta,varalpha,rho,gam,bet,R,Rnet,borrowlim,psi,sig,Epsi,Vlogpsi,Elogpsi,ben,chi,Vlogzeta,CVzetapow,Epsipow,Elogpsipow,Vlogpsipow,Vpsipow,CVpsipow,chitild,rhopref
real(8)     :: simKY, ssbend1,ssbend2,gmuc,gwage,gass,gbc,gannprem,gannprem1,gsurprob,gpsi,git,gben,ghbar,ghbaradj
real(8)		:: ctax,ltax0,ltax1,ltax2,ktax,err_w,err_c,err_h,err_y
real(8), dimension(Twork)		    :: vareps,adjvareps,unemppos,varzeta,cofvarzeta
real(8)     :: Pk0zero(ngpa), Elogk0pos, Vlogk0pos, Elogk0neg, Vlogk0neg,CORRlogk0apos,CORRlogk0aneg,k0amount
real(8)		:: gamG,sigG,chiG,betG,borrowlimG,EpsiG,ElogpsiG,VlogpsiG,VlogzetaG,CVzetapowG,err_cG,err_hG,err_yG,err_wG,EpsipowG,ElogpsipowG,VlogpsipowG,VpsipowG,CVpsipowG,chitildG,rhoprefG

!OTHER
character(len=1)   filesep
real(8), dimension(Ttot)     :: surprob, annprem
real(8), dimension(Ttot)     :: popsize

!GLOBALS TO STORE RANDOM NUMBER GENERATIONS
REAL(long), DIMENSION(nsim,Twork)   :: zrand,erand,hrand,fTrand
REAL(long), DIMENSION(nsim)         :: fPrand,arand,krand1,krand2

!GLOBALS TO STORE SIMULATION RESULTS
REAL(8), DIMENSION(nsim, Ttot)	    :: ysim,csim,mucsim,ynetsim,cgrosim
REAL(8), DIMENSION(nsim, Twork)	    :: esim,zsim,wsim,hsim,mulsim,hbarsim,bsim,htotsim,fsim,taxsim
REAL(8), DIMENSION(nsim)            :: asim
REAL(8), DIMENSION(nsim,Tret)       :: psim
REAL(8), DIMENSION(nsim, Ttot+1)	:: ksim,pnetsim
INTEGER, DIMENSION(nsim, Twork)	    :: esimI,zsimI,hbarsimI,fTsimI,fsimI
INTEGER, DIMENSION(nsim)	        :: fPsimI,asimI,psimI
REAL(8), DIMENSION(nsim*Twork)		:: kvec

!GLOBALS TO STORE MOMENTS FROM MODEL
real(8), dimension(Ttot)            :: Ec,Ek,Pk0neg,Elogc,Elogk,Vlogc,Vlogk,Vk,GINIk
real(8), dimension(Twork)           :: Eh,Phpos,Ehpos,Phft,Ehft,Ew,Ew_hpos,Ey,Elogh,Elogh_ft,Elogw,Elogw_hpos,Elogy,Elogc_hpos,Elogh_kpos,Eloghtot_kpos,Elogw_kpos,Elogw_khpos,Elogk_hpos
real(8), dimension(Twork)           :: Ehtot,Ehbar,Eloghtot,Eloghbar,Vhbar,Vhtot,Vloghtot,Vloghbar,COVwhtot_hpos,COVchtot_hpos,COVhhbar_hpos
real(8), dimension(Twork)           :: Vh,Vlogh,Vlogh_ft,Vlogw,Vlogw_hpos,Vlogc_hpos,Vlogy,COVwh_hpos,COVcw_hpos,COVch_hpos,COVcw,COVkw,COVkw_hpos,COVkh_hpos,COVkhtot_hpos
real(8), dimension(Tret)            :: Ep,Elogp
real(8), dimension(7,Twork)         :: PCh,PCk,PCc,PCw,PCy

!GLOBALS TO STORE DATA
real(8), dimension(36)            :: dataEh,dataElogh,dataVh,dataCOVwh,dataPk0neg,dataElogc,dataVc,dataCOVcw,dataCOVch
real(8), dimension(36)            :: stdevEh,stdevElogh,stdevVh,stdevCOVwh,stdevPk0neg,stdevElogc,stdevVc,stdevCOVcw,stdevCOVch
integer, dimension(36)            :: NEh,NElogh,NVh,NCOVwh,NPk0neg,NElogc,NVc,NCOVcw,NCOVch


!ESTIMATION GLOBALS
integer					:: nparam,nmoments,objeval,ndfls,gibs
real(8), allocatable	:: paramguess(:),paramout(:),paramscale(:),paramlb(:),paramub(:),diagweight(:),nobsvec(:)

!$OMP THREADPRIVATE(gannprem,gannprem1,gsurprob,gwage,gpsi,gass,gbc,ghbar,gben,ghbaradj)

END MODULE Globals
