SUBROUTINE Grids

USE Parameters
USE Globals
USE Procedures

IMPLICIT NONE

INTEGER         :: it,ia,iz,ie,ip,info,if2,iazefh, ih,ifP,ifT
REAL(8)         :: lxa,lxb,lxc,lfa,lfb,lfc,lm,lfval,lfvec(2),lzgridpar(2),ltol,lva,zetaslope
REAL(8), EXTERNAL           :: GOLDEN, FnGridTrans, FnGridFixedEffect, FnGridPrefHet, FnGridPrefHetTrans,FnTax
EXTERNAL                    :: FnZGrid


!Transitory shocks
adjvareps = max(vareps - err_w,1.0e-7)

DO git = 1,Twork
    lxa = 1.0e-8
    lxb = 1.0
    CALL MNBRAK(lxa,lxb,lxc,lfa,lfb,lfc,FnGridTrans)
    lfval = GOLDEN(lxa,lxb,lxc,FnGridTrans,1.0e-2_8,lm)
    lfval = FnGridTrans(lm)
END DO
    
!Persistent component
varz(1) = vareta
DO it = 2, Twork
  varz(it) = (rho**2)*varz(it-1) + vareta
END DO  

if (ngpz==2) ltol = 1.0e-8
if (ngpz>2) ltol = 1.0e-6

lzgridpar(1) = 1.2
lzgridpar(2) = 0.5    
CALL hybrd1 ( FnZGrid,2,lzgridpar,lfvec,ltol,info)
CALL FnZGrid(2,lzgridpar,lfvec,info)
IF (INFO .ne. 1) THEN
    IF(Display>=1) THEN
        write(*,*) ' z grid failed first time around'
    END IF 
    IF (Display>=2) THEN            
        write(*,*) info
        write(*,*) varz
        write(*,*) varzapprox
        write(*,*) rho
        write(*,*) rhozapprox
    END IF
    lzgridpar(1) = 1.5
    lzgridpar(2) = 5.0    
    CALL hybrd1 ( FnZgrid,2,lzgridpar,lfvec,ltol, info)
    CALL FnZGrid(2,lzgridpar,lfvec,info)
    IF (INFO ==1) write(*,*) ' z grid worked second time around'
    IF (INFO .ne. 1) write(*,*) ' z grid did not converge second time either'
END IF

!Fixed Effect
IF (ngpa==1) THEN
	agrid = 0.0
	adist = 1.0

ELSE IF(EqualProbFEGrid==0) THEN
	lxa = 1.0e-8
	lxb = 1.0
	CALL MNBRAK(lxa,lxb,lxc,lfa,lfb,lfc,FnGridFixedEffect)
	lfval = GOLDEN(lxa,lxb,lxc,FnGridFixedEffect,1.0e-2_8,lm)
	lfval = FnGridFixedEffect(lm)
ELSE
	adist = 1.0/ngpa
	CALL InvNormPoints (ngpa,agrid) 	!by symmetery, Ea = 0
	lva = DOT_PRODUCT(adist,agrid**2.0) - DOT_PRODUCT(adist,agrid)**2.0
	agrid = sqrt(varalpha)*agrid/sqrt(lva)
	!write(*,*) ' a mean: ',DOT_PRODUCT(adist,agrid)
	!write(*,*) ' a var: ',DOT_PRODUCT(adist,agrid**2.0) - DOT_PRODUCT(adist,agrid)**2.0
	!write(*,*) ' a grid: ', agrid
	!write(*,*) ' a dist: ', adist
END IF


!Wages
DO it = 1,Twork
    avwage(it) = 0.0
    DO ia = 1,ngpa
    DO iz = 1,ngpz
    DO ie = 1,ngpe
        wgrid(it,ia,iz,ie) = dexp(kappa(it) +egrid(it,ie) + zgrid(it,iz) + agrid(ia))
        avwage(it) = avwage(it) + wgrid(it,ia,iz,ie)*zdist(it,iz)*edist(it,ie)*adist(ia)
    END DO
    END DO
    END DO
END DO

!Normalize Average Wage to 1
wgrid  =  wgrid/DOT_PRODUCT(avwage,popsize(1:Twork)/SUM(popsize(1:Twork)))
avwage = avwage/DOT_PRODUCT(avwage,popsize(1:Twork)/SUM(popsize(1:Twork)))



!Preference Heterogeneity: permanent
IF(ngpfP>1) THEN         
	IF(Vlogpsi>0.0) THEN
	    lxa = 1.0e-8
	    lxb = 1.0
	    CALL MNBRAK(lxa,lxb,lxc,lfa,lfb,lfc,FnGridPrefHet)
	    lfval = GOLDEN(lxa,lxb,lxc,FnGridPrefHet,1.0e-2_8,lm)
	    lfval = FnGridPrefHet(lm)
	ELSE
	    fPgrid(:) = Epsi
	    fPdist(:) = 1.0/real(ngpfP)
	END IF
ELSE
    fPgrid(1) = Epsi
    fPdist(1) = 1.0		
END IF

!Preference Heterogeneity: transitory
IF(ngpfT>1) THEN

	zetaslope = 0.0 !for now have constant variance
	
	cofvarzeta(1) = CVzetapow
	varzeta(1) = (1.0-gamG**2.0)*log(cofvarzeta(1)**2.0+1.0)
	
	DO git = 2,Twork
		cofvarzeta(git) = cofvarzeta(git-1) + zetaslope
		varzeta(git) = max((1.0-gamG**2.0)*log(cofvarzeta(git)**2.0+1.0),0.0)
	END DO

	DO git = 1,Twork
	    IF(varzeta(git)>0.0) THEN
	    	lxa = 1.0e-8
		    lxb = 1.0
		    CALL MNBRAK(lxa,lxb,lxc,lfa,lfb,lfc,FnGridPrefHetTrans)
		    lfval = GOLDEN(lxa,lxb,lxc,FnGridPrefHetTrans,1.0e-2_8,lm)
		    lfval = FnGridPrefHetTrans(lm)
		ELSE
			fTgrid(git,:) = 1.0
			fTdist(git,:) = 1.0/real(ngpfT)		
		END IF
	END DO

ELSE
    fTgrid(:,1) = 1.0
    fTdist(:,1) = 1.0		
END IF


!combine permanent and transitpry pref het into combined grid and dist
DO git = 1,Twork
	if2 = 1
	DO ifP = 1,ngpfP
	DO ifT = 1,ngpfT
		fgrid(git,if2) = fPgrid(ifP) * fTgrid(git,ifT)
		IF(git==1) THEN
			find(ifP,ifT) = if2
			fPind(if2) = ifP
			fTind(if2) = ifT				
		END IF
		if2 = if2+1
	END DO
	END DO
END DO


!Hours shocks (iid case)
IF(ngphbar==1) THEN
	hgrid = 1.0
	hdist = 1.0
ELSE IF (ngphbar==2) THEN
	hgrid(1) = 0.8
	hgrid(ngphbar) = 1.0
	DO it = 1,Twork
		DO ifP = 1,ngpfP
			hdist(it,1,ifP) = (1.0+rhopref*fPgrid(ifP)/Epsi)*unemppos(it)
			hdist(it,ngphbar,ifP) = 1.0 - hdist(it,1,ifP)
		END DO
	END DO
	hdist = max(hdist,min(hdist,1.0),0.0)
ELSE IF (ngphbar==3) THEN
	hgrid(1) = 0.8 - 0.17
	hgrid(2) = 0.8 + 0.17
	hgrid(ngphbar) = 1.0	
	DO it = 1,Twork
	DO ifP = 1,ngpfP
		hdist(it,1,ifP) = (1.0+rhopref*fPgrid(ifP)/Epsi)*unemppos(it)/2.0
		hdist(it,2,ifP) = hdist(it,1,ifP)
		hdist(it,ngphbar,ifP) = 1.0 - hdist(it,1,ifP)*2.0
	END DO
	END DO
	hdist = max(hdist,min(hdist,1.0),0.0)
ELSE
	write(*,*) 'cant handle more than 3 points for hbar for now'
	stop
END IF

!Pensions
CALL rtsecSSParam(0.01_8,0.2_8,1.0e-3_8)
pgrid = pgrid
DO it = 1,Tret
DO ip = 1,ngpp
	pnetgrid(it,ip) = pgrid(it,ip) - FnTax(pgrid(it,ip))
END DO
END DO

!Natural borrowing limits
!Last period
DO ip = 1, ngpp
	nblret(Tret,ip) = -annprem(Twork+tret)*pnetgrid(Tret,ip)/Rnet + cmin
END DO
!Retired
DO it = Tret-1,1,-1
	DO ip = 1, ngpp
        nblret(it,ip) = annprem(Twork+it)*(nblret(it+1,ip) - pnetgrid(it,ip))/Rnet + cmin
    END DO
END DO
!Working
DO ia = 1,ngpa
    nbl(Twork,ia) = annprem(Twork)*(nblret(1,pind(1,ia)) - wgrid(Twork,ia,1,1)*hgrid(1) +FnTax(wgrid(Twork,ia,1,1)*hgrid(1)))/Rnet + cmin
    DO it = Twork-1,1,-1
        nbl(it,ia) = annprem(it)*(nbl(it+1,ia) - wgrid(it,ia,1,1)*hgrid(1) +FnTax(wgrid(it,ia,1,1)*hgrid(1)))/Rnet + cmin
    END DO
END DO

!Assets: 0 to amax - exponentially spaced grid - pexpgrid is parametr
DO it = 1,Twork
    DO ia = 1,ngpa
		CALL PowerSpacedGrid (ngpk,kgridparam,max(borrowlim,nbl(it,ia)),kmax,kgrid(it,:,ia))
    END DO        
END DO
DO it = 1,Tret
    DO ip = 1,ngpp
		CALL PowerSpacedGrid (ngpk,kgridparam,max(borrowlim,nblret(it,ip)),kmax,kgridret(it,:,ip))
    END DO
END DO




!Grids for parallel loop
iazefh =0
DO ia = 1,ngpa
DO iz = 1,ngpz
DO ie = 1,ngpe
DO if2 = 1,ngpf
DO ih = 1,ngphbar
    iazefh = iazefh+1
    aloopgrid(iazefh)=ia 
    zloopgrid(iazefh)=iz 
    eloopgrid(iazefh)=ie  
	floopgrid(iazefh)=if2 
    hloopgrid(iazefh)=ih
    
END DO
END DO
END DO
END DO
END DO
IF (Display>=2) write(*,*) 'Finished forming grids'


END SUBROUTINE Grids