MODULE Parameters
IMPLICIT NONE
SAVE

integer,parameter :: long = 8

!DIRECTORIES
!character(len=*), parameter	:: ParamOutputBaseDir = "Output/"
character(len=*), parameter	:: ParamOutputBaseDir = "Output/"
character(len=*), parameter	:: InputDir = "Input/"
character(len=*), parameter	:: ParamDataDir = "Data/"
character(len=*), parameter	:: ParamInputWageEstDir = "WageData/"

!OPTIONS
integer,parameter  ::  RunOnLinux           = 1
integer,parameter  ::  Display              = 2 !0,1,2
integer,parameter  ::  DoBootstraps			= 0	
integer,parameter  ::  SaveSimulations     	= 0
integer,parameter  ::  DoEstimation     	= 0
integer,parameter  ::  ReadParameterFile    = 1
integer,parameter  ::  UseBoundConstraints	= 0
integer,parameter  ::  MatchKYRatio         = 0	!see options below if ==1
integer,parameter  ::  MatchAggHours        = 0 !if doing estimation, only active if not estimating Epsi
integer,parameter  ::  AnnuityMarkets       = 0
integer,parameter  ::  NoDeathProb          = 0
integer,parameter  ::  InitWealthDist       = 1
integer,parameter  ::  CorrAlphaK0       	= 1
integer,parameter  ::  EqualProbFEGrid     	= 1

!WEALTH TARGET FOR BETA
integer,parameter  ::  MatchUsualKY         = 0	!over all ages
integer,parameter  ::  MatchMeanK         	= 0	!over working ages
integer,parameter  ::  MatchMedianK         = 0	!at Twork
integer,parameter  ::  MatchP75K         	= 1 !at Twork

!MEASUREMENT ERROR OPTIONS

!LEISURE UTILTIY OPTIONS
integer,parameter  ::  LeisUtilPower        = 0	!cant handle progressive taxation
integer,parameter  ::  HourUtilPower        = 1	!for now this is the only one that can handle hours constraints and progressive taxation
integer,parameter  ::  LeisUtilExpon        = 0 !cant handle estimation or progressive taxation
integer,parameter  ::  HourUtilExpon        = 0 !cant handle estimation or progressive taxation

!PENSION OPTIONS
integer,parameter  ::  FlatPension          = 0
integer,parameter  ::  LinearPension        = 0
integer,parameter  ::  ConcavePension       = 1
integer,parameter  ::  MatchTotPenBen		= 0 !instead of average pension benefit
integer,parameter  ::  PensionFEOnly		= 1 !instead of function of final persistent
integer,parameter  ::  ScaleBendPoints		= 0

!ESTIMATION OPTIONS
integer,parameter  ::  MatchMeansInLogs		= 1 !cant handle levels for consumption yet
integer,parameter  ::  EquivalenceScale		= 4 !1: RAW, 2: DP, 3: SU, 4: OECD-MODIFIED, 5: OECD, 6: SQRT
integer,parameter  ::  UseDiagWeight		= 1	!1 for inverse variance weighting, 0 for no weighting 

integer,parameter  ::  EstimateGamma		= 1
integer,parameter  ::  EstimateSigma		= 1
integer,parameter  ::  EstimateMeanPref		= 0	!use 0 for matching internallly Ehours
integer,parameter  ::  EstimateVarPref		= 1
integer,parameter  ::  EstimateVarPrefTrans	= 0
integer,parameter  ::  EstimateRhoPref		= 0
integer,parameter  ::  EstimateBeta			= 0
integer,parameter  ::  EstimateChi			= 0
integer,parameter  ::  EstimateBorrowLim	= 1
integer,parameter  ::  EstimateErrC			= 0
integer,parameter  ::  EstimateErrY			= 0
integer,parameter  ::  EstimateErrH			= 0

integer,parameter  ::  MatchMeanCon			= 1
integer,parameter  ::  MatchMeanHour		= 1
integer,parameter  ::  MatchVarCon			= 1
integer,parameter  ::  MatchVarHour			= 1
integer,parameter  ::  MatchCovWageHour		= 0
integer,parameter  ::  MatchCovConWage		= 0
integer,parameter  ::  MatchCovConHour		= 0
integer,parameter  ::  MatchZeroNegWealth	= 0


real(8),parameter  		:: hminpos      = 0.0
real(8),parameter       :: hminft		= 0.3
real(8),parameter		:: cmin			= 1.0e-4	

! GRIDS DIMENSION - STATE VARIABLES
integer,parameter :: ngpe = 5                   !transitory component: odd only
integer,parameter :: ngpz = 11                  !persistent component: odd only
integer,parameter :: ngpa = 5                   !fixed effect: odd only
integer,parameter :: ngpk = 30                  !asset points

integer,parameter :: ngpfP = 3                   !pref het: permanent
integer,parameter :: ngpfT = 1                   !pref het: transitory
integer,parameter :: ngpf = ngpfP*ngpFT          !pref het

integer,parameter :: ngpp = ngpa*ngpz	        !pension points
integer,parameter :: ngphbar = 3             !hours constraint

!DEMOGRAPHIC PARAMETERS PARAMETERS
integer,parameter	:: Twork    = 36		!working years
integer,parameter   :: Tret     = 30        !retirement years
integer,parameter   :: Ttot     = Twork+Tret        !total years

!TARGET MOMENTS
real(8), parameter      :: targetKY				= 2.5	!this is KY as would be in GE Bewley economy
real(8), parameter      :: targetMeanK			= 1.86	!this is wealth /annual wages from data - ie only for working age population
real(8), parameter      :: targetMedianK		= 1.61  !this is wealth /annual wages from data - at final working age (>=36 in data)
real(8), parameter      :: targetP75K			= 3.52 	!this is wealth /annual wages from data - at final working age (>=36 in data)

real(8), parameter      :: targethours			= 0.44
real(8), parameter      :: targetSSBenToLabinc 	= 0.095		!only used if MactchTotPenBen==1	
real(8), parameter      :: targetSSAvBen		= 0.34		!this needs a better calibration

!PARAMETERS FOR GRID CONSTRUCTION
real(8), parameter   :: kgridparam 	= 0.3       !approaches linear as goes to 1, approaches L shaped as goes to 0
real(8), parameter   :: kmax  		= 30.0


!SIMULATION PARAMETERS	
integer, parameter	:: nsim = 50000 !10000	!50000 for estimation

!BOOTSTRAPS
integer, parameter	:: nbstraps = 39

END MODULE Parameters

