SUBROUTINE SetParameters

USE Parameters
USE Globals
USE Procedures

IMPLICIT NONE
CHARACTER(len=1000) :: charbs

!INTEREST RATE
R = 1.03       !annual gross interest rate

!PREFERENCE PARMETERS
gamG = 2.33
sigG = 2.65
 chiG = -0.59 ! <=1: 1 is standard case, 1/(1+sig) is linear, 0 is hpw only, <0 is strong aversion to unemployment
 chitildG = -(chiG-1.0)*(1.0+sigG)

rhoprefG = 1.2	!correlation of pref het with hbar shocks

if(LeisUtilPower==1 .or. HourUtilPower==1) then  
    EpsipowG = 0.4
	CVpsipowG = 0.82 !1.0
	CVzetapowG 	= 0.0 !0.05

	VpsipowG = (EpsipowG*CVpsipowG)**2.0
	VlogpsipowG = log(VpsipowG/(EpsipowG**2.0) +1.0)
	ElogpsipowG = log(EpsipowG) - VlogpsipowG/2.0
	ElogpsiG = (1.0-gamG)*ElogpsipowG - (gamG+sigG)*log(targethours)
	EpsiG = exp((1.0-gamG)*ElogpsipowG +((1.0-gamG)**2.0)*VlogpsipowG/2.0) / (targethours**(sigG+gamG))
! 	VlogpsiG = ((1.0-gam)**2.0)*VlogpsipowG
	VlogpsiG = ((1.0-gamG)**2.0)*VlogpsipowG

    VlogzetaG = (1.0-gamG**2.0)*log(CVzetapowG**2.0+1.0)

else if(LeisUtilExpon==1 .or. HourUtilExpon==1) then

!these two have to be checked
    VlogpsiG = 0.036
    VlogzetaG = 0.0
    ElogpsiG = 15.0
    EpsiG = exp(ElogpsiG+(VlogpsiG)/2.0)
	EpsipowG = (EpsiG * (targethours**(sigG+gamG)) ) **(1.0/(1.0-gamG))
end if


betG = 0.98

!UNEMPLOYMENT BENEFITS
ben = 0.0

!BORROWING LIMIT: SET TO VERY LARGE NEGATIVE NO. FOR NBL
IF(EstimateBorrowLim==1) borrowlimG =  -0.15
IF(EstimateBorrowLim==0) borrowlimG =  0.0

!INITIAL WEALTH PARAMETERS
!assume mass on zero, and then log normal, (use exper<=5 in data)
Pk0zero     = (/ 0.32, 0.28, 0.25, 0.15, 0.16 /)
Elogk0pos   = -2.4
Vlogk0pos   = 2.4
 CORRlogk0apos = 0.10
Elogk0neg   = -2.9
Vlogk0neg   = 2.1
 CORRlogk0aneg = 0.17
k0amount	= 1.0e-4	!if no dist, this is how much they start off with

!TAXES - PROPORTIONAL ON CONSUMPTION AND CAPITAL INCOME, PROGRESSIVE ON EARNINGS
ktax = 0.4
 ctax = 0.08

ltax0 = -0.00559	!transfer at zero income
ltax1 =  0.63768	!1-marginal tax rate at earnings=1
ltax2 = -0.13615	!amount of progressivity

Rnet = (1.0 +(1.0-ktax)*(R-1.0))

!MEASUREMENT ERROR
err_cG = 0.05
err_yG = 0.01
err_hG = 0.01
err_wG = err_hG + err_yG


!READ GUESSES TO OVERIDE PARAMETERS
IF (ReadParameterFile==1) THEN
	OPEN(1, FILE = InputDir // trim(InputParamFile))
	READ(1,*) OutputBaseDir
	READ(1,*) DataBaseDir
	READ(1,*) InputWageEstBaseDir
	READ(1,*) gamG
	READ(1,*) sigG
	READ(1,*) EpsipowG
	READ(1,*) CVpsipowG
	READ(1,*) CVzetapowG
	READ(1,*) borrowlimG
	READ(1,*) chiG	
	READ(1,*) rhoprefG	
 	READ(1,*) err_yG	
 	READ(1,*) err_hG	
 	READ(1,*) err_cG	
 	READ(1,*) betG	
	CLOSE(1)

	IF(EstimateBorrowLim==0) borrowlimG =  0.0	!to avoid having separate guess files
	IF(EstimateVarPrefTrans==0) CVzetapowG =  0.0	!to avoid having separate guess files

	VpsipowG = (EpsipowG*CVpsipowG)**2.0
	VlogpsipowG = log(VpsipowG/(EpsipowG**2.0) +1.0)
	ElogpsipowG = log(EpsipowG) - VlogpsipowG/2.0
	ElogpsiG = (1.0-gamG)*ElogpsipowG - (gamG+sigG)*log(targethours)
	EpsiG = exp((1.0-gamG)*ElogpsipowG +((1.0-gamG)**2.0)*VlogpsipowG/2.0) / (targethours**(sigG+gamG))
	VlogpsiG = ((1.0-gamG)**2.0)*VlogpsipowG

	VlogzetaG = (1.0-gamG**2.0)*log(CVzetapowG**2.0+1.0)

 	chitildG = -(chiG-1.0)*(1.0+sigG)
	err_wG = err_hG + err_yG

ELSE
	OutputBaseDir 		= ParamOutputBaseDir
	DataBaseDir 		= ParamDataDir
	InputWageEstBaseDir = ParamInputWageEstDir
END IF

IF (DoBootstraps==0) THEN
	OutputDir = trim(OutputBaseDir)
ELSE IF (DoBootstraps==1) THEN
	write(charbs,*) gibs
	charbs = adjustl(charbs)
	OutputDir = trim(OutputBaseDir) // 'bs' // trim(charbs)  // filesep	
END IF


IF(DoEstimation==0) THEN
	gam = gamG
	sig = sigG
	
    Epsipow = EpsipowG
    CVpsipow = CVpsipowG	
    Vpsipow = VpsipowG	
	Vlogpsipow = VlogpsipowG
	Elogpsipow = 	ElogpsipowG
	Elogpsi = ElogpsiG
	Epsi = EpsiG
  	Vlogpsi = VlogpsiG
	
	CVzetapow = CVzetapowG	
	Vlogzeta = VlogzetaG	

	bet = betG
	chi = chiG
	chitild = chitildG
	rhopref = rhoprefG
	
	borrowlim = borrowlimG
	err_c = err_cG
	err_y = err_yG
	err_h = err_hG
	err_w = err_wG
	Epsipow = EpsipowG
END IF


END SUBROUTINE SetParameters