SUBROUTINE Simulate

USE Parameters
USE Globals
USE Procedures

IMPLICIT NONE

INTEGER		:: itemp(1)
INTEGER		:: in,it
REAL(8)		:: lhbaradj
REAL(8), EXTERNAL	:: FnTax

!preference heterogeneity
IF (ngpfP==1) fPsimI(:) = 1
IF (ngpfP>1) 	CALL DiscreteDist(nsim,fPsimI,ngpfP,fPdist,fPrand)


!fixed effect
CALL DiscreteDist(nsim,asimI,ngpa,adist,arand)
asim = agrid(asimI)

!initial wages
CALL DiscreteDist(nsim,zsimI(:,1),ngpz,zdist(1,:),zrand(:,1))
CALL DiscreteDist(nsim,esimI(:,1),ngpe,edist(1,:),erand(:,1))
zsim(:,1) = zgrid(1,zsimI(:,1))
esim(:,1) = egrid(1,esimI(:,1))

!initial assets
IF (InitWealthDist==1) THEN
	SELECT CASE (CorrAlphaK0)
		CASE(0)
		    DO in = 1,nsim
		        CALL DiscreteDist1(itemp(1),2,(/sum(Pk0zero)/real(ngpa),1.0-sum(Pk0zero)/real(ngpa)/),krand1(in))
		        IF (itemp(1)==2) THEN !positive
		            ksim(in,1) = exp(krand2(in)*sqrt(Vlogk0pos) + Elogk0pos)
					ksim(in,1) = min(ksim(in,1),kmax)
		        ELSE !zero or negative
		            ksim(in,1) = -exp(krand2(in)*sqrt(Vlogk0neg) + Elogk0neg)
		            ksim(in,1) = max(ksim(in,1),kgrid(1,1,asimI(in)))
		        END IF
		    END DO

		CASE(1)
		    DO in = 1,nsim
		        CALL DiscreteDist1(itemp(1),2,(/Pk0zero(asimI(in)),1.0-Pk0zero(asimI(in))/),krand1(in))
		        IF (itemp(1)==2) THEN !positive
		            ksim(in,1) = exp(krand2(in)*sqrt(Vlogk0pos*(1.0 - CORRlogk0apos**2.0)) &
		 							+ Elogk0pos + asim(in)*CORRlogk0apos*sqrt(Vlogk0pos/varalpha))
					ksim(in,1) = min(ksim(in,1),kmax)
		        ELSE !zero or negative
		            ksim(in,1) = -exp(krand2(in)*sqrt(Vlogk0neg*(1.0 - CORRlogk0aneg**2.0)) &
		 							+ Elogk0neg + asim(in)*CORRlogk0aneg*sqrt(Vlogk0neg/varalpha))
		            ksim(in,1) = max(ksim(in,1),kgrid(1,1,asimI(in)))
		        END IF
			END DO
	END SELECT

ELSE
    ksim(:,1) = k0amount
END IF

    
!working life
DO it = 1,Twork
    if (Display>=2) write(*,*) 'Simulating at age ', it				
    
    !$OMP PARALLEL DO PRIVATE(lhbaradj)
    DO in = 1,nsim
 
		IF (ngpfT==1) fTsimI(in,it) = 1
		IF (ngpfT>1) CALL DiscreteDist1(fTsimI(in,it),ngpfT,fTdist(it,:),fTrand(in,it))
		fsimI(in,it) = find(fPsimI(in),fTsimI(in,it))
		fsim(in,it) = fgrid(it,fsimI(in,it))


        IF(it>1) THEN
			CALL DiscreteDist1(zsimI(in,it), ngpz, ztrans(it-1,zsimI(in,it-1),:),zrand(in,it))
	        CALL DiscreteDist1(esimI(in,it), ngpe, edist(it,:),erand(in,it))
		END IF

		CALL DiscreteDist1(hbarsimI(in,it),ngphbar,hdist(it,:,fPsimI(in)),hrand(in,it))

        IF (it==Twork) THEN
            psimI(in) = pind(zsimI(in,Twork),asimI(in))
        END IF
        
        zsim(in,it) = zgrid(it,zsimI(in,it))
        esim(in,it) = egrid(it,esimI(in,it))
        wsim(in,it) = wgrid(it,asimI(in),zsimI(in,it),esimI(in,it))
        hbarsim(in,it) = hgrid(hbarsimI(in,it))
		lhbaradj = hbarsim(in,it)**(chi*(1+sig))
		bsim(in,it) = ben*(1.0-hbarsim(in,it))

        IF(ksim(in,it) == kgrid(it,1,asimI(in))) THEN
            csim(in,it) = con(it,1,asimI(in),zsimI(in,it),esimI(in,it),fsimI(in,it),hbarsimI(in,it))
            hsim(in,it) = hour(it,1,asimI(in),zsimI(in,it),esimI(in,it),fsimI(in,it),hbarsimI(in,it))
			htotsim(in,it)	= hsim(in,it)*hbarsim(in,it)
            ysim(in,it) = wsim(in,it)*htotsim(in,it) + bsim(in,it)   
            ynetsim(in,it)  = ysim(in,it) -FnTax(ysim(in,it))
            ksim(in,it+1) = Rnet*ksim(in,it)/annprem(it)+ ynetsim(in,it) - csim(in,it)*(1.0+ctax)
	        mucsim(in,it)   = csim(in,it)**(-gam)

	        if(LeisUtilPower==1) mulsim(in,it) = fsim(in,it)*(1.0-hsim(in,it))**(-sig)
	        if(LeisUtilExpon==1) mulsim(in,it) = fsim(in,it)*dexp(sig*(1.0-hsim(in,it)))
	        if(HourUtilPower==1) mulsim(in,it) = lhbaradj*fsim(in,it)*(hsim(in,it)**(sig-ltax2))/(ltax1*((wsim(in,it)*hbarsim(in,it))**ltax2))
	        if(HourUtilExpon==1) mulsim(in,it) = fsim(in,it)*dexp(sig*(hsim(in,it)))

        ELSE    
            CALL LinInterp1 (ngpk,kgrid(it,:,asimI(in)),con(it,:,asimI(in),zsimI(in,it),esimI(in,it),fsimI(in,it),hbarsimI(in,it)),ksim(in,it),csim(in,it))
            mucsim(in,it)   = csim(in,it)**(-gam)
            mulsim(in,it)   = hbarsim(in,it)*wsim(in,it)*mucsim(in,it)/(1.0+ctax)
        
            if(LeisUtilPower==1) hsim(in,it)     = 1.0 - (mulsim(in,it)/fsim(in,it))**(-1.0/sig)
            if(LeisUtilExpon==1) hsim(in,it)     = 1.0 - (1.0/sig)*dlog(mulsim(in,it)/fsim(in,it)) 
            if(HourUtilPower==1) hsim(in,it)     = (mulsim(in,it)*ltax1*((wsim(in,it)*hbarsim(in,it))**ltax2)/fsim(in,it))**(1.0/(sig-ltax2)) * (hbarsim(in,it)**(-chi*(1+sig)/(sig-ltax2)))
            if(HourUtilExpon==1) hsim(in,it)     = (1.0/sig)*dlog(mulsim(in,it)/fsim(in,it))
                    
            hsim(in,it)     = min(max(hsim(in,it),0.0_8),1.0)
			htotsim(in,it)	= hsim(in,it)*hbarsim(in,it)
            ysim(in,it)     = wsim(in,it)*htotsim(in,it) + bsim(in,it)
            ynetsim(in,it)  = ysim(in,it) - FnTax(ysim(in,it))
            ksim(in,it+1)   = Rnet*ksim(in,it)/annprem(it)+ ynetsim(in,it) - csim(in,it)*(1.0+ctax)
        END IF
        
        IF (it<Twork) THEN    
            IF (ksim(in,it+1)<kgrid(it+1,1,asimI(in)))THEN
                IF (Display>=2 .and. ksim(in,it+1)<kgrid(it+1,1,asimI(in))-1.0e-3) write(*,*) 'consumption interpolation failed: in=',in,' it=',it
                ksim(in,it+1)  = kgrid(it+1,1,asimI(in))
                CALL LinInterp1 (ngpk,kgrid(it,:,asimI(in)),hour(it,:,asimI(in),zsimI(in,it),esimI(in,it),fsimI(in,it),hbarsimI(in,it)),ksim(in,it),hsim(in,it))
                hsim(in,it)     = min(max(hsim(in,it),0.0_8),1.0)
				htotsim(in,it)	= hsim(in,it)*hbarsim(in,it)
                ysim(in,it) = wsim(in,it)*htotsim(in,it) + bsim(in,it)
	            ynetsim(in,it)  = ysim(in,it) - FnTax(ysim(in,it))
                csim(in,it) = (Rnet*ksim(in,it)/annprem(it)+ ynetsim(in,it) - ksim(in,it+1)) / (1.0+ctax)
		        mucsim(in,it)   = csim(in,it)**(-gam)

				if(LeisUtilPower==1) mulsim(in,it) = fsim(in,it)*(1.0-hsim(in,it))**(-sig)
				if(LeisUtilExpon==1) mulsim(in,it) = fsim(in,it)*dexp(sig*(1.0-hsim(in,it)))
				if(HourUtilPower==1) mulsim(in,it) = lhbaradj*fsim(in,it)*(hsim(in,it)**(sig-ltax2))/(ltax1*((wsim(in,it)*hbarsim(in,it))**ltax2))
				if(HourUtilExpon==1) mulsim(in,it) = fsim(in,it)*dexp(sig*(hsim(in,it)))

            END IF        
        ELSE IF (it==Twork) THEN
            IF (ksim(in,it+1)<kgridret(1,1,psimI(in)))THEN
                IF (Display>=2 .and. ksim(in,it+1)<kgridret(1,1,psimI(in))-1.0e-3) write(*,*) 'consumption interpolation failed: in=',in,' it=',it
                ksim(in,it+1)  = kgridret(1,1,psimI(in))
                CALL LinInterp1 (ngpk,kgrid(it,:,asimI(in)),hour(it,:,asimI(in),zsimI(in,it),esimI(in,it),fsimI(in,it),hbarsimI(in,it)),ksim(in,it),hsim(in,it))
                hsim(in,it)     = min(max(hsim(in,it),0.0_8),1.0)
				htotsim(in,it)	= hsim(in,it)*hbarsim(in,it)
                ysim(in,it) = wsim(in,it)*htotsim(in,it) + bsim(in,it)
	            ynetsim(in,it)  = ysim(in,it) - FnTax(ysim(in,it))
                csim(in,it) = (Rnet*ksim(in,it)/annprem(it)+ ynetsim(in,it) - ksim(in,it+1)) / (1.0+ctax)
		        mucsim(in,it)   = csim(in,it)**(-gam)

		        if(LeisUtilPower==1) mulsim(in,it) = fsim(in,it)*(1.0-hsim(in,it))**(-sig)
		        if(LeisUtilExpon==1) mulsim(in,it) = fsim(in,it)*dexp(sig*(1.0-hsim(in,it))) !mistake corrected
				if(HourUtilPower==1) mulsim(in,it) = lhbaradj*fsim(in,it)*(hsim(in,it)**(sig-ltax2))/(ltax1*((wsim(in,it)*hbarsim(in,it))**ltax2))
		        if(HourUtilExpon==1) mulsim(in,it) = fsim(in,it)*dexp(sig*(hsim(in,it)))
            END IF        
        END IF

        IF((csim(in,it)<=0.0) .or. isnan(csim(in,it))) THEN
			IF (Display>=2) write(*,*) 'bad consumption',in,' it=',it
        END IF          
  		
		taxsim(in,it) = ysim(in,it) - ynetsim(in,it)
    END DO
    !$OMP END PARALLEL DO
    
END DO

!retirement
DO it = 1,Tret
    if (Display>=2) write(*,*) 'Simulating at age ', Twork+it				
    
    !$OMP PARALLEL DO
    DO in = 1,nsim
        psim(in,it) = pgrid(it,psimI(in))
        pnetsim(in,it) = pnetgrid(it,psimI(in))
		ysim(in,Twork+it) = psim(in,it)
        ynetsim(in,Twork+it) = pnetsim(in,it)
        
        IF(ksim(in,Twork+it) == kgridret(it,1,psimI(in))) THEN
            csim(in,Twork+it) = conret(it,1,psimI(in))
            ksim(in,Twork+it+1) = Rnet*ksim(in,Twork+it)/annprem(Twork+it)+ ynetsim(in,Twork+it) - csim(in,Twork+it)*(1.0+ctax)
        ELSE    
            CALL LinInterp1 (ngpk,kgridret(it,:,psimI(in)),conret(it,:,psimI(in)),ksim(in,Twork+it),csim(in,Twork+it))
            ksim(in,Twork+it+1) = Rnet*ksim(in,Twork+it)/annprem(Twork+it) + ynetsim(in,Twork+it) - csim(in,Twork+it)*(1.0+ctax)
        END IF
            
        IF (it<Tret) THEN
            IF (ksim(in,Twork+it+1)<kgridret(it+1,1,asimI(in)))THEN
                ksim(in,Twork+it+1) = kgridret(it+1,1,asimI(in))
                csim(in,Twork+it) = (Rnet*ksim(in,Twork+it)/annprem(Twork+it) - ksim(in,Twork+it+1) + ynetsim(in,Twork+it)) / (1.0+ctax)
            END IF        
        ELSE IF (it==Tret) THEN
            IF (ksim(in,Twork+it+1) .ne. 0)THEN
                ksim(in,Twork+it+1) = 0
                csim(in,Twork+it) = (Rnet*ksim(in,Twork+it)/annprem(Twork+it) - ksim(in,Twork+it+1) + ynetsim(in,Twork+it) ) / (1.0+ctax)
            END IF             
        END IF
        
        mucsim(in,it)   = csim(in,it)**(-gam)
        
    END DO
    !$OMP END PARALLEL DO
    
END DO
	cgrosim = csim*(1.0+ctax)
if (any(isnan(ksim))) then
	write(*,*) 'nan in ksim'
end if

END SUBROUTINE Simulate